/*---------------------------------------------------------------------------*\
 * Copyright (C) 2000 AVM GmbH. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
 * ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
 * with the use or performance of this software. See the
 * GNU General Public License for further details.
\*---------------------------------------------------------------------------*/

#include "CapiBase.h"
#include "capi20.h"
#include "osImport.h"

#define WAIT_TIME_FOR_BUSY  50 // msec

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

static void *thread_callback (void *pBase) {
    dhead ("thread_callback", DEBUG_CCAPI20_Base);
    dassert (pBase != 0);
    ((CCAPI20_Base *)pBase)->GetMessageThread();
    return 0;
}


/*===========================================================================*\
    CCAPI20_Base - Constructor
\*===========================================================================*/

CCAPI20_Base::CCAPI20_Base (void) {
    dhead ("CCAPI20_Base-Constructor", DEBUG_CCAPI20_Base);
    applID     = IllegalApplID;
    threadStop = 0;
}


/*===========================================================================*\
    CCAPI20_Base - Destructor
\*===========================================================================*/

CCAPI20_Base::~CCAPI20_Base (void) {
    dhead ("CCAPI20_Base-Destructor", DEBUG_CCAPI20_Base);
    dassert (applID == IllegalApplID);
}


/*===========================================================================*\
    CCAPI20_Base::IsInstalled
\*===========================================================================*/

void CCAPI20_Base::IsInstalled (void) {
    dhead ("CCAPI20_Base::IsInstalled", DEBUG_CCAPI20_Base);
    IsInstalledComplete ((c_info)capi20_isinstalled());
}


/*===========================================================================*\
    CCAPI20_Base::Register
\*===========================================================================*/

void CCAPI20_Base::Register (tUInt /*MsgBufSize*/, tUInt MaxB3Connection, tUInt MaxB3Blks,
                             tUInt MaxSizeB3) {
    dhead ("CCAPI20_Base::Register", DEBUG_CCAPI20_Base);
    // MsgBufSize is not needed under Linux
    dassert (MaxB3Connection > 0);
    dassert (MaxB3Blks >= 2);
    dassert (MaxB3Blks <= 7);
    dassert (MaxSizeB3 > 0);
    dassert (applID == IllegalApplID);

    c_info error = (c_info)capi20_isinstalled();
    if (CapiInfoIsError (error)) {
        dprint ("IsInstalled return=%x ", error);
        RegisterComplete (error);
        return;
    }
    unsigned aID = IllegalApplID;
    error        = (c_info)capi20_register (MaxB3Connection, MaxB3Blks, MaxSizeB3, &aID);
    dassert (aID < (c_word)-1);
    dwarning (aID != 0);
    dinfo (error == 0, " error=%x ", error);
    if ((aID != IllegalApplID) && CapiInfoIsSuccess (error)) {
        if (!sem_init (&semaphore, 0, 0)) {
            applID     = aID;
            threadStop = 0;
            if (!pthread_create (&thread, 0, thread_callback, this)) {
                // everything is initialised successfully
                RegisterComplete (ci_NoError);
                return;
            }
            dwarning (0);
            sem_destroy (&semaphore);
        } else {
            dwarning (0);
        }
        error  = (c_info)capi20_release (aID);
        // what can we do if release failed?
        dassert (error == ci_NoError);
        applID = IllegalApplID;
        error  = ci_Reg_OSResourceErr;
    }
    RegisterComplete (error);
}


/*===========================================================================*\
    CCAPI20_Base::Release
\*===========================================================================*/

void CCAPI20_Base::Release (void) {
    dhead ("CCAPI20_Base::Release", DEBUG_CCAPI20_Base);
    dassert (applID != IllegalApplID);
    threadStop++;
    sem_post (&semaphore);
    c_info error = (c_info)capi20_release (applID);
    dassert (error == ci_NoError);
    applID = IllegalApplID;
    threadStop++;
    if (threadStop == 3) {
        sem_destroy (&semaphore);
        ReleaseComplete (error);
    }
}


/*===========================================================================*\
    CCAPI20_Base::PutMessage
\*===========================================================================*/

void CCAPI20_Base::PutMessage (c_message Message) {
    dhead ("CCAPI20_Base::PutMessage", DEBUG_CCAPI20_Base);
    dassert (Message != 0);
    dassert (applID != IllegalApplID);
    // TODO: Busy abfangen !!!!!!!!!!!!!!!!!!!!!
    dprint ("ApplID=%x\n", applID);
    //dprintBuf (Message, CapiMsg_GetMsgLen (Message));
    c_info error = (c_info)capi20_put_message (applID, Message);
    switch (error) {
    case ci_Msg_Busy:
    case ci_Msg_SendQueueFull:
        dassert (0);
        // TODO: Call Timer;
        //break;
    default:
        dwarning (0);
    case ci_NoError:
        PutMessageComplete (error);
        break;
    }
}


/*===========================================================================*\
    CCAPI20_Base::GetMessage
\*===========================================================================*/

void CCAPI20_Base::GetMessage (void) {
    dhead ("CCAPI20_Base::GetMessage", DEBUG_CCAPI20_Base);
    dassert (applID != IllegalApplID);
    // activate the thread
    #ifndef NDEBUG
        int sv = 0;
        sem_getvalue (&semaphore, &sv);
        dassert (sv == 0); // GetMessage must Complete before it can called again!
    #endif
    sem_post (&semaphore);
}


/*===========================================================================*\
    CCAPI20_Base::GetManufacturer
\*===========================================================================*/

void CCAPI20_Base::GetManufacturer (tUInt) {
    dhead ("CCAPI20_Base::GetManufacturer", DEBUG_CCAPI20_Base);
    /*----- TODO: ist noch nicht implementiert -----*/
    dassert (0);
}


/*===========================================================================*\
    CCAPI20_Base::GetVersion
\*===========================================================================*/

void CCAPI20_Base::GetVersion (tUInt) {
    dhead ("CCAPI20_Base::GetVersion", DEBUG_CCAPI20_Base);
    /*----- TODO: ist noch nicht implementiert -----*/
    dassert (0);
}


/*===========================================================================*\
    CCAPI20_Base::GetSerialNumber
\*===========================================================================*/

void CCAPI20_Base::GetSerialNumber (tUInt) {
    dhead ("CCAPI20_Base::GetSerialNumber", DEBUG_CCAPI20_Base);
    /*----- TODO: ist noch nicht implementiert -----*/
    dassert (0);
}


/*===========================================================================*\
    CCAPI20_Base::GetProfile
\*===========================================================================*/

void CCAPI20_Base::GetProfile (tUInt Controller) {
    dhead ("CCAPI20_Base::GetProfile", DEBUG_CCAPI20_Base);
    cp_getprofile prof;
    GetProfileComplete (&prof, (c_info)capi20_get_profile (Controller, (tUByte *)&prof));
}


/*===========================================================================*\
    CCAPI20_Base::GetMessageThread
\*===========================================================================*/

void CCAPI20_Base::GetMessageThread (void) {
    dhead ("CCAPI20_Base::GetMessageThread", DEBUG_CCAPI20_Base);
    c_word aID = applID;
    dwarning (applID != 0);
    c_info error;
    tUChar *pBuf;
    sem_wait (&semaphore);
    while (!threadStop) {
        if (!capi20_waitformessage (aID, 0) && !threadStop) {
            error = (c_info)capi20_get_message (aID, &pBuf);
            switch (error) {
            case ci_Msg_ReceiveQueueEmpty:
                // Nothing to do
                break;
            case ci_Msg_Busy:
                // TODO: usleep (WAIT_TIME_FOR_BUSY);
                break;
            default:
                GetMessageComplete (pBuf, error);
                if (!threadStop) {
                    sem_wait (&semaphore);
                }
                break;
            }
        }
    }
    threadStop++;
    if (threadStop == 3) {
        sem_destroy (&semaphore);
        ReleaseComplete (ci_NoError);
    }
}

/*===========================================================================*\
\*===========================================================================*/

