/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _CAPICHAN_H_
#define _CAPICHAN_H_

#include "CapiLay.h"
#include "CBTimer.h"

/*===========================================================================*\
\*===========================================================================*/

class CCAPI20_Channel : protected CCAPI20_Layer {
public:
    enum State {
        cs_Released,                // inital state
        cs_Releasing,               // back to the initial state
        cs_Registering,             // build up to listen state
        cs_Registered,              // listen or make call state
        cs_D_ConnectRequest,
        cs_D_ConnectIndication,
        cs_D_ConnectPending,
        cs_D_Connected,
        cs_B_ConnectPending,
        cs_Connected,
        cs_B_DisconnectPending,
        cs_B_Disconnected,
        cs_D_DisconnectPending      // leads to cs_Registered
    };

    CCAPI20_Channel (void);
    virtual ~CCAPI20_Channel (void);

    State GetState (void);
    c_dword GetController (void);
    c_dword GetPLCI (void);
    c_dword GetNCCI (void);

    void SetNCPI (c_struct NCPI);
    tBool AcceptIncomingCall (void);
    // AcceptIncommingCall have to be called before AlertReq and ConnectResp
    // return:  vTrue  - allow to accept incoming call
    //          vFalse - don't accept incoming call

    /*-----------------------------------------------------------------------*\
        Anmeldung
    \*-----------------------------------------------------------------------*/
    void Register (void);
    void Release (void);
    virtual void Register_Complete (c_info ErrorCode) = 0;
    virtual void Release_Complete (c_info ErrorCode) = 0;

    /*-----------------------------------------------------------------------*\
        CAPI - Requests
    \*-----------------------------------------------------------------------*/
    void AlertReq      (c_struct AdditionalInfo);
    void ConnectReq    (c_dword  Controller, cp_connectreq *Params);
    void DataB3Req     (cp_datab3 *Params);
    void DisconnectReq (void);
    void ListenReq     (c_dword Controller, cp_listenreq *Params);

    /*-----------------------------------------------------------------------*\
        virtual CAPI - Confirmations
    \*-----------------------------------------------------------------------*/
    virtual void DataB3Conf (cp_datab3conf * /*Params*/) {}
    virtual void ListenConf (c_dword /*Controller*/, c_info /*Info*/) {}

    /*-----------------------------------------------------------------------*\
        virtual CAPI - Indications
    \*-----------------------------------------------------------------------*/
    virtual void ConnectInd         (c_dword /*Controller*/, cp_connectind * /*Params*/) {}
    virtual void ConnectActiveInd   (c_struct /*ConnectedNumber*/) {}
    virtual void ConnectB3ActiveInd (c_struct /*NCPI*/) {}
    virtual void DataB3Ind          (cp_datab3 * /*Params*/) {}
    virtual void DisconnectB3Ind    (cp_disconnectb3ind * /*Params*/) {}
    virtual void DisconnectInd      (c_info /*Reason*/) {}
    virtual void InfoInd            (cp_infoind * /*Params*/) {}

    /*-----------------------------------------------------------------------*\
        CAPI - Responses
    \*-----------------------------------------------------------------------*/
    void ConnectResp  (cp_connectresp *Params);
    void DataB3Resp   (c_word DataHandle);

private:
    /*-----------------------------------------------------------------------*\
        virtuelle Funktionen der CCAPI20_ChannelForManager
    \*-----------------------------------------------------------------------*/
    void REGISTER_COMPLETE (c_info ErrorCode);
    void RELEASE_COMPLETE (c_info ErrorCode);

    void CONNECT_CONF (c_word Messagenumber, c_dword PLCI, c_info Info);
    void CONNECT_B3_CONF (c_word Messagenumber, c_dword NCCI, c_info Info);
    void DATA_B3_CONF (c_word Messagenumber, c_dword NCCI, cp_datab3conf *Params);
    void DISCONNECT_B3_CONF (c_word Messagenumber, c_dword NCCI, c_info Info);
    void DISCONNECT_CONF (c_word Messagenumber, c_dword PLCI, c_info Info);
    void LISTEN_CONF (c_word Messagenumber, c_dword Controller, c_info Info);

    tBool CONNECT_IND (c_word Messagenumber, c_dword PLCI, cp_connectind *Params);
    tBool CONNECT_ACTIVE_IND (c_word Messagenumber, c_dword PLCI, c_struct ConnectedNumber);
    tBool CONNECT_B3_IND (c_word Messagenumber, c_dword NCCI, c_struct NCPI);
    tBool CONNECT_B3_ACTIVE_IND (c_word Messagenumber, c_dword NCCI, c_struct NCPI);
    tBool DATA_B3_IND (c_word Messagenumber, c_dword NCCI, cp_datab3 *Params);
    tBool DISCONNECT_B3_IND (c_word Messagenumber, c_dword NCCI, cp_disconnectb3ind *Params);
    tBool DISCONNECT_IND (c_word Messagenumber, c_dword PLCI, c_info Reason);
    tBool INFO_IND (c_word Messagenumber, c_dword Address, cp_infoind *Params);

    /*---------------------------------------------------------------------------*\
        interne Funktionen
    \*---------------------------------------------------------------------------*/
    void HandleSupplementaryServices (c_supplserv Function, tUInt structLen, c_byte *structData);
    void SetState (State s);
    void SetController (c_dword cntrl);
    void SetPLCI (c_dword PLCI);
    void SetNCCI (c_dword NCCI);

    /*-----------------------------------------------------------------------*\
       interne Variablen
    \*-----------------------------------------------------------------------*/
    CCallBackTimer         BChannelTimer;
    State                  curState;
    c_dword                controller;
    c_dword                plci;
    c_dword                ncci;
    c_struct               NCPIout;
    tBool                  Initiator;
};

/*===========================================================================*\
    inline Funktionen
\*===========================================================================*/

inline CCAPI20_Channel::State CCAPI20_Channel::GetState (void) {
    return curState;
}

inline c_dword CCAPI20_Channel::GetController (void) {
    return controller;
}

inline c_dword CCAPI20_Channel::GetPLCI (void) {
    return plci;
}

inline c_dword CCAPI20_Channel::GetNCCI (void) {
    return ncci;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CCAPI20_Channel::AlertReq (c_struct AdditionalInfo) {
    // assert (plci != IllegalPLCI);
    ALERT_REQ (GetNextMsgNr(), GetPLCI(), AdditionalInfo);
}

inline void CCAPI20_Channel::ListenReq (c_dword Controller, cp_listenreq *Params) {
    LISTEN_REQ (GetNextMsgNr(), Controller, Params);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CCAPI20_Channel::SetState (CCAPI20_Channel::State s) {
    curState = s;
}

inline void CCAPI20_Channel::SetController (c_dword cntrl) {
    controller = cntrl;
}

inline void CCAPI20_Channel::SetPLCI (c_dword PLCI) {
    plci = PLCI;
}

inline void CCAPI20_Channel::SetNCCI (c_dword NCCI) {
    ncci = NCCI;
}

/*===========================================================================*\
\*===========================================================================*/

#endif
