/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _CAPIMSG_H_
#define _CAPIMSG_H_

#include "CapiBase.h"
#include "c2msgdef.h"
#include "DynStore.h"
#include <stdarg.h>

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define REGISTER_STATE_RELEASED     0
#define REGISTER_STATE_REGISTERING  1
#define REGISTER_STATE_REGISTERED   2
#define REGISTER_STATE_RELEASING    3
#define REGISTER_INSTALL_TESTING    4

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

extern cp_connectresp   cpV_ConnectResp_Ignore;
extern cp_connectb3resp cpV_ConnectB3Resp_Reject;

/*===========================================================================*\
\*===========================================================================*/

class CCAPI20_MsgBase : private CCAPI20_Base {
public:
    CCAPI20_MsgBase (void);
    virtual ~CCAPI20_MsgBase (void);

    c_word GetApplID (void);
    tUInt GetRegState (void);

    /*-----------------------------------------------------------------------*\
        CAPI - REGISTER, RELEASE, GET_PROFILE
    \*-----------------------------------------------------------------------*/
    void INSTALLED (void);
    void REGISTER (tUInt MaxB3Connection, tUInt MaxB3Blks, tUInt MaxSizeB3);
    void RELEASE  (void);

    void GET_PROFILE (tUInt Controller);

    virtual void INSTALLED_COMPLETE (c_info /*Info*/) {}
    virtual void REGISTER_COMPLETE (c_info ErrorCode) = 0;
    virtual void RELEASE_COMPLETE (c_info ErrorCode) = 0;

    virtual void GET_PROFILE_COMPLETE (cp_getprofile * /*pProfile*/, c_info /*ErrorCode*/) {}

    /*-----------------------------------------------------------------------*\
        CAPI - Requests
    \*-----------------------------------------------------------------------*/
    void ALERT_REQ                         (c_word   Messagenumber,
                                            c_dword  PLCI,
                                            c_struct AdditionalInfo);

    void CONNECT_REQ                       (c_word         Messagenumber,
                                            c_dword        Controller,
                                            cp_connectreq *Params);

    void CONNECT_B3_REQ                    (c_word   Messagenumber,
                                            c_dword  PLCI,
                                            c_struct NCPI);

    void DATA_B3_REQ                       (c_word     Messagenumber,
                                            c_dword    NCCI,
                                            cp_datab3 *Params);

    void DISCONNECT_B3_REQ                 (c_word   Messagenumber,
                                            c_dword  NCCI,
                                            c_struct NCPI);

    void DISCONNECT_REQ                    (c_word   Messagenumber,
                                            c_dword  PLCI);

    void FACILITY_REQ                      (c_word       Messagenumber,
                                            c_dword      Address,
                                            cp_facility *Params);

    void INFO_REQ                          (c_word      Messagenumber,
                                            c_dword     Address,
                                            cp_inforeq *Params);

    void LISTEN_REQ                        (c_word        Messagenumber,
                                            c_dword       Controller,
                                            cp_listenreq *Params);

    void MANUFACTURER_REQ                  (c_word           Messagenumber,
                                            c_dword          Controller,
                                            cp_manufacturer *Params);

    void RESET_B3_REQ                      (c_word   Messagenumber,
                                            c_dword  NCCI,
                                            c_struct NCPI);

    void SELECT_B_PROTOCOL_REQ             (c_word        Messagenumber,
                                            c_dword       PLCI,
                                            cp_bprotocol *Params);

    /*-----------------------------------------------------------------------*\
        virtual CAPI - Confirmation
    \*-----------------------------------------------------------------------*/

    virtual void ALERT_CONF                (c_word  /*Messagenumber*/,
                                            c_dword /*PLCI*/,
                                            c_info  /*Info*/) {}

    virtual void CONNECT_CONF              (c_word  /*Messagenumber*/,
                                            c_dword /*PLCI*/,
                                            c_info  /*Info*/) {}

    virtual void CONNECT_B3_CONF           (c_word  /*Messagenumber*/,
                                            c_dword /*NCCI*/,
                                            c_info  /*Info*/) {}

    virtual void DATA_B3_CONF              (c_word          /*Messagenumber*/,
                                            c_dword         /*NCCI*/,
                                            cp_datab3conf * /*Params*/) {}

    virtual void DISCONNECT_B3_CONF        (c_word  /*Messagenumber*/,
                                            c_dword /*NCCI*/,
                                            c_info  /*Info*/) {}

    virtual void DISCONNECT_CONF           (c_word  /*Messagenumber*/,
                                            c_dword /*PLCI*/,
                                            c_info  /*Info*/) {}

    virtual void FACILITY_CONF             (c_word            /*Messagenumber*/,
                                            c_dword           /*Address*/,
                                            cp_facilityconf * /*Params*/) {}

    virtual void INFO_CONF                 (c_word  /*Messagenumber*/,
                                            c_dword /*Address*/,
                                            c_info  /*Info*/) {}

    virtual void LISTEN_CONF               (c_word  /*Messagenumber*/,
                                            c_dword /*Controller*/,
                                            c_info  /*Info*/) {}

    virtual void MANUFACTURER_CONF         (c_word            /*Messagenumber*/,
                                            c_dword           /*Controller*/,
                                            cp_manufacturer * /*Params*/) {}

    virtual void RESET_B3_CONF             (c_word  /*Messagenumber*/,
                                            c_dword /*NCCI*/,
                                            c_info  /*Info*/) {}

    virtual void SELECT_B_PROTOCOL_CONF    (c_word  /*Messagenumber*/,
                                            c_dword /*PLCI*/,
                                            c_info  /*Info*/) {}

    /*-----------------------------------------------------------------------*\
        virtual CAPI - Indications
    \*-----------------------------------------------------------------------*/

    virtual tBool CONNECT_IND               (c_word         Messagenumber,
                                             c_dword        PLCI,
                                             cp_connectind *Params);

    virtual tBool CONNECT_ACTIVE_IND        (c_word   Messagenumber,
                                             c_dword  PLCI,
                                             c_struct ConnectedNumber);

    virtual tBool CONNECT_B3_IND            (c_word   Messagenumber,
                                             c_dword  NCCI,
                                             c_struct NCPI);

    virtual tBool CONNECT_B3_ACTIVE_IND     (c_word   Messagenumber,
                                             c_dword  NCCI,
                                             c_struct NCPI);

    virtual tBool CONNECT_B3_T90_ACTIVE_IND (c_word   Messagenumber,
                                             c_dword  NCCI);

    virtual tBool DATA_B3_IND               (c_word     Messagenumber,
                                             c_dword    NCCI,
                                             cp_datab3 *Params);

    virtual tBool DISCONNECT_B3_IND         (c_word              Messagenumber,
                                             c_dword             NCCI,
                                             cp_disconnectb3ind *Params);

    virtual tBool DISCONNECT_IND            (c_word  Messagenumber,
                                             c_dword PLCI,
                                             c_info  Reason);

    virtual tBool FACILITY_IND              (c_word       Messagenumber,
                                             c_dword      Address,
                                             cp_facility *Params);

    virtual tBool INFO_IND                  (c_word      Messagenumber,
                                             c_dword     Address,
                                             cp_infoind *Params);

    virtual tBool MANUFACTURER_IND          (c_word           Messagenumber,
                                             c_dword          Controller,
                                             cp_manufacturer *Params);

    virtual tBool RESET_B3_IND              (c_word   Messagenumber,
                                             c_dword  NCCI,
                                             c_struct NCPI);

    /*-----------------------------------------------------------------------*\
        CAPI - Responses
    \*-----------------------------------------------------------------------*/
    void CONNECT_RESP                      (c_word          Messagenumber,
                                            c_dword         PLCI,
                                            cp_connectresp *Params);

    void CONNECT_ACTIVE_RESP               (c_word  Messagenumber,
                                            c_dword PLCI);

    void CONNECT_B3_RESP                   (c_word            Messagenumber,
                                            c_dword           NCCI,
                                            cp_connectb3resp *Params);

    void CONNECT_B3_ACTIVE_RESP            (c_word  Messagenumber,
                                            c_dword NCCI);

    void CONNECT_B3_T90_ACTIVE_RESP        (c_word  Messagenumber,
                                            c_dword NCCI);

    void DATA_B3_RESP                      (c_word  Messagenumber,
                                            c_dword NCCI,
                                            c_word  DataHandle);

    void DISCONNECT_B3_RESP                (c_word  Messagenumber,
                                            c_dword NCCI);

    void DISCONNECT_RESP                   (c_word  Messagenumber,
                                            c_dword PLCI);

    void FACILITY_RESP                     (c_word       Messagenumber,
                                            c_dword      Address,
                                            cp_facility *Params);

    void INFO_RESP                         (c_word  Messagenumber,
                                            c_dword Address);

    void MANUFACTURER_RESP                 (c_word           Messagenumber,
                                            c_dword          Controller,
                                            cp_manufacturer *Params);

    void RESET_B3_RESP                     (c_word  Messagenumber,
                                            c_dword NCCI);

    /*-----------------------------------------------------------------------*\
        Error Handling
    \*-----------------------------------------------------------------------*/
    virtual void HandlePutMessageError (c_message cmsg, c_info ErrorCode);

private:
    /*-----------------------------------------------------------------------*\
        Virtuelle Funktionen von CCAPI20_Base
    \*-----------------------------------------------------------------------*/
    void IsInstalledComplete  (c_info ErrorCode);

    void RegisterComplete (c_info ErrorCode);

    void ReleaseComplete (c_info ErrorCode);

    void PutMessageComplete (c_info ErrorCode);

    void GetMessageComplete (c_message ReturnMessage, c_info ErrorCode);

    void GetProfileComplete (cp_getprofile *pProfile, c_info ErrorCode);

    /*-----------------------------------------------------------------------*\
        Interne Funtkionen
    \*-----------------------------------------------------------------------*/
    void AllocAndPutMsg (c_byte Command, c_byte SubCommand, c_word Messagenumber,
                         c_dword Address, char *format, ...);
    void FillMessage (c_message cmsg, c_byte Command, c_byte SubCommand, c_word Messagenumber,
                      c_dword Address, char *format, va_list param);
    void FreeCurPutMsg (void);
    void EmptyPutMsgbufferQueue (void);

    /*-----------------------------------------------------------------------*\
        Interne Klassen und Variablen
    \*-----------------------------------------------------------------------*/
    CDynStoreQueue PutMsgbufferQueue;
    CProtectionLock Protect;

    c_byte *CurPutMessage;
    tUInt   RegisterState;
};


/*===========================================================================*\
    inline Funktionen der Klasse CCAPI20_MsgBase
\*===========================================================================*/

inline c_word CCAPI20_MsgBase::GetApplID (void) {
    return CCAPI20_Base::GetApplID();
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUInt CCAPI20_MsgBase::GetRegState (void) {
    return RegisterState;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CCAPI20_MsgBase::GET_PROFILE (tUInt Controller) {
    GetProfile (Controller);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CCAPI20_MsgBase::GetProfileComplete (cp_getprofile *pProfile, c_info ErrorCode) {
    GET_PROFILE_COMPLETE (pProfile, ErrorCode);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CCAPI20_MsgBase::FreeCurPutMsg (void) {
    Protect.Begin();
    c_byte *toDel = CurPutMessage;
    CurPutMessage = 0;
    Protect.End();
    PutMsgbufferQueue.Free (toDel);
}


/*===========================================================================*\
    CCAPI20_MsgBase: Alle CAPI - Requests
\*===========================================================================*/

inline void CCAPI20_MsgBase::ALERT_REQ (c_word Messagenumber, c_dword PLCI, c_struct AdditionalInfo) {
    AllocAndPutMsg (CAPI_ALERT, CAPI_REQ, Messagenumber, PLCI, "4", AdditionalInfo);
}

inline void CCAPI20_MsgBase::CONNECT_REQ (c_word Messagenumber, c_dword Controller, cp_connectreq *Params) {
    AllocAndPutMsg (CAPI_CONNECT, CAPI_REQ, Messagenumber, Controller, "244252224443", Params->CIPValue,
                    Params->CalledPartyNumber, Params->CallingPartyNumber, 0,
                    (CalcCapiStructLen (Params->bp.B1Configuration) +
                     CalcCapiStructLen (Params->bp.B2Configuration) +
                     CalcCapiStructLen (Params->bp.B3Configuration) + 3 * sizeof (c_word)),
                    Params->bp.B1Protocol,
                    Params->bp.B2Protocol, Params->bp.B3Protocol, Params->bp.B1Configuration,
                    Params->bp.B2Configuration, Params->bp.B3Configuration, 0);
}

inline void CCAPI20_MsgBase::CONNECT_B3_REQ (c_word Messagenumber, c_dword PLCI, c_struct NCPI) {
    AllocAndPutMsg (CAPI_CONNECT_B3, CAPI_REQ, Messagenumber, PLCI, "4", NCPI);
}

inline void CCAPI20_MsgBase::DATA_B3_REQ (c_word Messagenumber, c_dword NCCI, cp_datab3 *Params) {
    #ifdef C_PLATFORM_64
        AllocAndPutMsg (CAPI_DATA_B3, CAPI_REQ, Messagenumber, NCCI, "32226", 0, Params->DataLength,
                        Params->DataHandle, Params->Flags, Params->Data);
    #else
        AllocAndPutMsg (CAPI_DATA_B3, CAPI_REQ, Messagenumber, NCCI, "3222", Params->Data, Params->DataLength,
                        Params->DataHandle, Params->Flags);
    #endif
}

inline void CCAPI20_MsgBase::DISCONNECT_B3_REQ (c_word Messagenumber, c_dword NCCI, c_struct NCPI) {
    AllocAndPutMsg (CAPI_DISCONNECT_B3, CAPI_REQ, Messagenumber, NCCI, "4", NCPI);
}

inline void CCAPI20_MsgBase::DISCONNECT_REQ (c_word Messagenumber, c_dword PLCI) {
    AllocAndPutMsg (CAPI_DISCONNECT, CAPI_REQ, Messagenumber, PLCI, "1", 0);
}

inline void CCAPI20_MsgBase::FACILITY_REQ (c_word Messagenumber, c_dword Address, cp_facility *Params) {
    AllocAndPutMsg (CAPI_FACILITY, CAPI_REQ, Messagenumber, Address, "24", Params->FacilitySelector,
                    Params->FacilityParam);
}

inline void CCAPI20_MsgBase::INFO_REQ (c_word Messagenumber, c_dword Address, cp_inforeq *Params) {
    c_word AddInfoLen = (c_word)(CalcCapiStructLen (Params->BChannelInformation) +
                                 CalcCapiStructLen (Params->KeypadFacility) +
                                 CalcCapiStructLen (Params->UserUserData) +
                                 CalcCapiStructLen (Params->FacilityDataArray));
    AllocAndPutMsg (CAPI_INFO, CAPI_REQ, Messagenumber, Address, "454444", Params->CalledPartyNumber, AddInfoLen,
                    Params->BChannelInformation, Params->KeypadFacility, Params->UserUserData, Params->FacilityDataArray);
}


inline void CCAPI20_MsgBase::LISTEN_REQ (c_word Messagenumber, c_dword Controller, cp_listenreq *Params) {
    AllocAndPutMsg (CAPI_LISTEN, CAPI_REQ, Messagenumber, Controller, "33341", Params->InfoMask, Params->CIPMask, 0,
                    Params->CallingPartyNumber, 0);
}


inline void CCAPI20_MsgBase::MANUFACTURER_REQ (c_word Messagenumber, c_dword Controller, cp_manufacturer *Params) {
    AllocAndPutMsg (CAPI_MANUFACTURER, CAPI_REQ, Messagenumber, Controller, "35334", Params->ManuID,
                    (c_word)(CalcCapiStructLen (Params->ManuData) + 6), Params->Class, Params->Function, Params->ManuData);
}


inline void CCAPI20_MsgBase::RESET_B3_REQ (c_word Messagenumber, c_dword NCCI, c_struct NCPI) {
    AllocAndPutMsg (CAPI_RESET_B3, CAPI_REQ, Messagenumber, NCCI, "4", NCPI);
}

inline void CCAPI20_MsgBase::SELECT_B_PROTOCOL_REQ (c_word Messagenumber, c_dword PLCI, cp_bprotocol *Params) {
    c_word BProtLen = (c_word)(6 + CalcCapiStructLen (Params->B1Configuration) +
                                   CalcCapiStructLen (Params->B2Configuration) +
                                   CalcCapiStructLen (Params->B3Configuration));
    AllocAndPutMsg (CAPI_SELECT_B_PROTOCOL, CAPI_REQ, Messagenumber, PLCI, "5222444", BProtLen, Params->B1Protocol,
                    Params->B2Protocol, Params->B3Protocol, Params->B1Configuration, Params->B2Configuration,
                    Params->B3Configuration);
}


/*===========================================================================*\
    CCAPI20_MsgBase: Alle CAPI - Indications
\*===========================================================================*/

inline tBool CCAPI20_MsgBase::CONNECT_IND (c_word Messagenumber, c_dword PLCI, cp_connectind *) {
    CONNECT_RESP (Messagenumber, PLCI, &cpV_ConnectResp_Ignore);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::CONNECT_ACTIVE_IND (c_word Messagenumber, c_dword PLCI, c_struct) {
    CONNECT_ACTIVE_RESP (Messagenumber, PLCI);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::CONNECT_B3_IND (c_word Messagenumber, c_dword NCCI, c_struct) {
    CONNECT_B3_RESP (Messagenumber, NCCI, &cpV_ConnectB3Resp_Reject);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::CONNECT_B3_ACTIVE_IND (c_word Messagenumber, c_dword NCCI, c_struct) {
    CONNECT_B3_ACTIVE_RESP (Messagenumber, NCCI);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::CONNECT_B3_T90_ACTIVE_IND (c_word Messagenumber, c_dword NCCI) {
    CONNECT_B3_T90_ACTIVE_RESP (Messagenumber, NCCI);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::DATA_B3_IND (c_word Messagenumber, c_dword NCCI, cp_datab3 *Params) {
    DATA_B3_RESP (Messagenumber, NCCI, Params->DataHandle);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::DISCONNECT_B3_IND (c_word Messagenumber, c_dword NCCI, cp_disconnectb3ind *) {
    DISCONNECT_B3_RESP (Messagenumber, NCCI);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::DISCONNECT_IND (c_word Messagenumber, c_dword PLCI, c_info) {
    DISCONNECT_RESP (Messagenumber, PLCI);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::FACILITY_IND (c_word Messagenumber, c_dword Address, cp_facility *Params) {
    //cp_facility RespParams = { Params->FacilitySelector, 0 };
    //FACILITY_RESP (Messagenumber, Address, &RespParams);
    FACILITY_RESP (Messagenumber, Address, Params);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::INFO_IND (c_word Messagenumber, c_dword Address, cp_infoind *) {
    INFO_RESP (Messagenumber, Address);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::MANUFACTURER_IND (c_word Messagenumber, c_dword Controller, cp_manufacturer *Params) {
    MANUFACTURER_RESP (Messagenumber, Controller, Params);
    return vTrue;
}

inline tBool CCAPI20_MsgBase::RESET_B3_IND (c_word Messagenumber, c_dword NCCI, c_struct) {
    RESET_B3_RESP (Messagenumber, NCCI);
    return vTrue;
}

/*===========================================================================*\
    CCAPI20_MsgBase: Nearly All CAPI - Responses
\*===========================================================================*/

inline void CCAPI20_MsgBase::CONNECT_ACTIVE_RESP (c_word Messagenumber, c_dword PLCI) {
    AllocAndPutMsg (CAPI_CONNECT_ACTIVE, CAPI_RESP, Messagenumber, PLCI, "");
}

inline void CCAPI20_MsgBase::CONNECT_B3_RESP (c_word Messagenumber, c_dword NCCI, cp_connectb3resp *Params) {
    AllocAndPutMsg (CAPI_CONNECT_B3, CAPI_RESP, Messagenumber, NCCI, "24", Params->Reject, Params->NCPI);
}

inline void CCAPI20_MsgBase::CONNECT_B3_ACTIVE_RESP (c_word Messagenumber, c_dword NCCI) {
    AllocAndPutMsg (CAPI_CONNECT_B3_ACTIVE, CAPI_RESP, Messagenumber, NCCI, "");
}

inline void CCAPI20_MsgBase::CONNECT_B3_T90_ACTIVE_RESP (c_word Messagenumber, c_dword NCCI) {
    AllocAndPutMsg (CAPI_CONNECT_B3_T90_ACTIVE, CAPI_RESP, Messagenumber, NCCI, "");
}

inline void CCAPI20_MsgBase::DATA_B3_RESP (c_word Messagenumber, c_dword NCCI, c_word DataHandle) {
    AllocAndPutMsg (CAPI_DATA_B3, CAPI_RESP, Messagenumber, NCCI, "2", DataHandle);
}

inline void CCAPI20_MsgBase::DISCONNECT_B3_RESP (c_word Messagenumber, c_dword NCCI) {
    AllocAndPutMsg (CAPI_DISCONNECT_B3, CAPI_RESP, Messagenumber, NCCI, "");
}

inline void CCAPI20_MsgBase::DISCONNECT_RESP (c_word Messagenumber, c_dword PLCI) {
    AllocAndPutMsg (CAPI_DISCONNECT, CAPI_RESP, Messagenumber, PLCI, "");
}

inline void CCAPI20_MsgBase::FACILITY_RESP (c_word Messagenumber, c_dword Address, cp_facility *Params) {
    AllocAndPutMsg (CAPI_FACILITY, CAPI_RESP, Messagenumber, Address, "24", Params->FacilitySelector,
                    Params->FacilityParam);
}

inline void CCAPI20_MsgBase::INFO_RESP (c_word Messagenumber, c_dword Address) {
    AllocAndPutMsg (CAPI_INFO, CAPI_RESP, Messagenumber, Address, "");
}

inline void CCAPI20_MsgBase::MANUFACTURER_RESP (c_word Messagenumber, c_dword Controller, cp_manufacturer *Params) {
    AllocAndPutMsg (CAPI_MANUFACTURER, CAPI_RESP, Messagenumber, Controller, "35334", Params->ManuID,
                    (c_word) (6 + CalcCapiStructLen (Params->ManuData)), Params->Class, Params->Function, Params->ManuData);
}

inline void CCAPI20_MsgBase::RESET_B3_RESP (c_word Messagenumber, c_dword NCCI) {
    AllocAndPutMsg (CAPI_RESET_B3, CAPI_RESP, Messagenumber, NCCI, "");
}


/*===========================================================================*\
\*===========================================================================*/

#endif
