/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _CHANNEL_H_
#define _CHANNEL_H_

#include "CapiChan.h"
#include "DataHand.h"
#include "MSNList.h"
#include "ConvCapi.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define MAX_TRANSFER_BUFFER_SIZE        CapiMaxDataSize
#define MAX_OUTSTANDING_FRAMES          CapiMaxOutstandingFrames

#define NCPI_OPTIONS_HighResolution     1

/*===========================================================================*\
\*===========================================================================*/

class CTransferChannel : private CCAPI20_Channel {
public:
    CTransferChannel (void);
    virtual ~CTransferChannel (void);

    State   CurState (void);
    tUShort CountPutQueueItems (void);
    tBool   IsPutQueueEmpty (void);
    tBool   IsPutQueueFull (void);

    void SetOwnNumber (tUChar *pMyNumber, tUByte Indicator = CALLNUM_INDICATOR_DEFAULT);
    void SetB1Parameters (c_b1prot B1Protocol, cu_B1Configuration *pB1Config = 0);
    void SetB2Parameters (c_b2prot B2Protocol);
    void SetB3Parameters (c_b3prot B3Protocol, cu_B3Configuration *pB3Config = 0);
    void SetNCPI (c_b3prot B3Protocol, cu_NCPI *pNCPI);

    void ListenAdd (tUInt Controller, tUInt CIPMask, tUInt countMSN, tUChar **MSNs);
    void ListenAdd (tUInt Controller, tUInt CIPMask = 0);                   // MSN = OwnNumber
    void ListenAdd (tUInt Controller, tUInt CIPMask, tUInt MainNumLen, tUInt DDILen, tUChar *firstMSN = 0, tUChar *lastMSN = 0); // For DDI
    void ListenRemove (tUInt Controller = 0);                               // 0 = all controllers
    virtual void ListenError (tUInt /*Controller*/, c_info /*Info*/) {}     // Controller == 0 => For all Controller
    void ShareMSNList (CCntrlMSNList *pSharedList);

    void MakeCall (tUShort Controller, tUChar *pTelNumber, c_cipvalue CIPValue);
    void AlertCall (void);
    void AnswerCall (tBool Accept);
    tBool PutData (tUByte *Data, tUShort DataLength, void *hDataID);        // vFalse => QueueFull and no PutComplete
    void GetDataComplete (tUShort DataHandle);
    void Disconnect (void);

    virtual void IncomingCall (tUInt Controller, c_cipvalue CIPValue, tUChar *pOppositeNumber, tUChar *pMyCalledNumber) = 0;
    virtual void DoConnecting (void) = 0;                                   // (ConnectReq)
    virtual void ConnectProceeding (void) = 0;                              // (ConnectActiveInd)
    virtual void IsConnected (cu_NCPI *pNCPI) = 0;
    virtual void PutDataComplete (void *hDataID, c_info Info) = 0;
    virtual void GetData (tUByte *Data, tUShort DataLength, tUShort DataHandle) = 0;
    virtual void DoDisconnecting (c_info ReasonB3, cu_NCPI *pNCPI) = 0;
    virtual void IsDisconnected (c_info Reason) = 0;
    virtual void IsReleased (c_info Info) = 0;

protected:
    void CheckListen (void);

private:
    /*-----------------------------------------------------------------------*\
        Virtuelle Funktionen von CCAPI20_Channel
    \*-----------------------------------------------------------------------*/
    void Register_Complete (c_info ErrorCode);
    void Release_Complete (c_info ErrorCode);
    void DataB3Conf (cp_datab3conf *Params);
    void ListenConf (c_dword Controller, c_info Info);
    void ConnectInd (c_dword Controller, cp_connectind *Params);
    void ConnectActiveInd (c_struct ConnectedNumber);
    void ConnectB3ActiveInd (c_struct NCPI);
    void DataB3Ind (cp_datab3 *Params);
    void DisconnectB3Ind (cp_disconnectb3ind *Params);
    void DisconnectInd (c_info Reason);
    void InfoInd (cp_infoind *Params);

    /*-----------------------------------------------------------------------*\
        Interne Funktionen
    \*-----------------------------------------------------------------------*/
    void ResetSendBlock (void);

    /*-----------------------------------------------------------------------*\
        Interne Variablen
    \*-----------------------------------------------------------------------*/
    CDataHandleAdmin SendBlock;
    CCntrlMSNList   *pMSNList;
    tBool            deleteMSNList;
    tUInt            DDINumberLen;
    tUChar          *szCallingNumber;

    c_dword    controller;
    c_cipvalue cipValue;
    c_b1prot   B1Prot;
    c_b2prot   B2Prot;
    c_b3prot   B3Prot;
    c_struct   B1Config;
    c_struct   B3Config;
    c_struct   sMSN;
    c_struct   CalledPartyNum;
};

/*===========================================================================*\
    Inline Funktionen
\*===========================================================================*/

inline CTransferChannel::State CTransferChannel::CurState (void) {
    return GetState();
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tUShort CTransferChannel::CountPutQueueItems (void) {
    return SendBlock.CurCount();
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tBool CTransferChannel::IsPutQueueEmpty (void) {
    return SendBlock.IsEmpty();
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline tBool CTransferChannel::IsPutQueueFull (void) {
    return SendBlock.IsFull();
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::SetOwnNumber (tUChar *pMyNumber, tUByte Indicator) {
    ConvUnion_CallingNumber (&sMSN, pMyNumber, Indicator);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::SetB1Parameters (c_b1prot B1Protocol, cu_B1Configuration *pB1Config) {
    B1Prot = B1Protocol;
    ConvUnion_B1Configuration (&B1Config, pB1Config, B1Protocol);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::SetB2Parameters (c_b2prot B2Protocol) {
    B2Prot = B2Protocol;
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::SetB3Parameters (c_b3prot B3Protocol, cu_B3Configuration *pB3Config) {
    B3Prot = B3Protocol;
    ConvUnion_B3Configuration (&B3Config, pB3Config, B3Protocol);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::SetNCPI (c_b3prot B3Protocol, cu_NCPI *pNCPI) {
    c_struct NCPIuser = 0;
    ConvUnion_NCPI (&NCPIuser, pNCPI, B3Protocol);
    CCAPI20_Channel::SetNCPI (NCPIuser);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::ListenAdd (tUInt Controller, tUInt CIPMask, tUInt countMSN, tUChar **MSNs) {
    // assert (CIPMask != 0);
    if (pMSNList) {
        pMSNList->Add (Controller, 0, CIPMask);
        pMSNList->AddMSN (Controller, countMSN, MSNs);
    }
    CheckListen();
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::AlertCall (void) {
    if (GetState() == cs_D_ConnectIndication) {
        DDINumberLen = 0;
        AlertReq (0);
    }
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::GetDataComplete (tUShort DataHandle) {
    DataB3Resp (DataHandle);
}

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

inline void CTransferChannel::Disconnect (void) {
    DisconnectReq();
}

/*===========================================================================*\
\*===========================================================================*/

#endif
