/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _MSNLIST_H_
#define _MSNLIST_H_

#include "CSUnions.h"
#include "SortList.h"

/*===========================================================================*\
    CCntrlMSNList
\*===========================================================================*/

class CCntrlMSNList : private CSortPointerList {
public:
    enum ctInfo {
        i_Okay             = 0,
        i_RejectCall       = 1,
        i_AcceptCall       = 2,
        i_DDICall          = 3,
        i_Wrn_NotAllAdded  = 4,
        i_Err_MemoryFull   = 5,
        i_Err_MemoryAccess = 6
    };

    CCntrlMSNList (void);

    // Set remove every previous entry for the controller. Add integrate the new values to the previous one.
    // Set and Add doesn't change anything for the entries from SetMSN and AddMSN and vice versa.
    tBool Set (tUInt Controller, tUInt InfoMask, tUInt CIPMask, tUInt MainNumLen = 0, tUInt DDILen = 0);
    tBool Add (tUInt Controller, tUInt InfoMask, tUInt CIPMask, tUInt MainNumLen = 0, tUInt DDILen = 0);

    ctInfo SetMSN (tUInt Controller, tUInt countMSNs, tUChar **MSNs);
    ctInfo SetMSN (tUInt Controller, tUChar *aMSN);
    ctInfo AddMSN (tUInt Controller, tUInt countMSNs, tUChar **MSNs);
    ctInfo AddMSN (tUInt Controller, tUChar *aMSN);
    // For DDI it is only allowed to specify a maximum of two MSNs as a range

    void Remove (tUInt Controller);
    void RemoveAll (void);

    ctInfo FindCallInfo (tUInt Controller, c_cipvalue CIPValue, c_struct MSN);
    ctInfo TestDDINumber (tUInt Controller, c_struct DDINumber, tUInt *pMainNumLen = 0);
    // *pIntPartLen only valid if return = i_AccpeptCall

    void ResetGetNextMask (void);
    tBool GetNextMask (tUInt *pController, tUInt *pInfoMask, tUInt *pCIPMask);  // vFalse == no more masks
    tBool GetMask (tUInt Controller, tUInt *pInfoMask, tUInt *pCIPMask);        // vFalse == Controller is not in the list
    tBool IsEmpty (void);
    tBool ExistController (tUInt Controller);

    tUInt GetDDINumberLen (tUInt Controller);

private:
    class CControllerInfo *pCurPos;
};

/*===========================================================================*\
    inline functions from CCntrlMSNList
\*===========================================================================*/

inline CCntrlMSNList::CCntrlMSNList (void) {
    ResetGetNextMask();
}

inline CCntrlMSNList::ctInfo CCntrlMSNList::SetMSN (tUInt Controller, tUChar *aMSN) {
    return SetMSN (Controller, 1, &aMSN);
}

inline CCntrlMSNList::ctInfo CCntrlMSNList::AddMSN (tUInt Controller, tUChar *aMSN) {
    return AddMSN (Controller, 1, &aMSN);
}

inline void CCntrlMSNList::RemoveAll (void) {
    RemoveAndDeleteAll();
}

inline void CCntrlMSNList::ResetGetNextMask (void) {
    pCurPos = 0;
}

inline tBool CCntrlMSNList::IsEmpty (void) {
    return CSortPointerList::IsEmpty();
}

inline tBool CCntrlMSNList::ExistController (tUInt Controller) {
    return (Find ((void *)Controller) != 0);
}

/*===========================================================================*\
\*===========================================================================*/

#endif
