/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "PLCIList.h"
#include "SortList.h"
#include "Protect.h"
#include "osImport.h"
#include <string.h>

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

class CPLCIListerElement : public CSortPListElement {
public:
    tUInt  Counter;
    tUByte Controller;
    tUByte PLCIField[32];

    CPLCIListerElement (void) {
        Counter    = 0;
        Controller = 0;
        memset (PLCIField, '\0', 32 * sizeof (tUByte));
    }

    tCompare Compare (void *RefCompData) {
        if (((tUByte)RefCompData) < Controller) {
            return vLower;
        } else if (((tUByte)RefCompData) == Controller) {
            return vEqual;
        }
        return vHigher;
    }
    void *GetRefCompData (void) {
        return (void *)Controller;
    }
};

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

static CSortPointerList *pPLCIList = 0;
static CProtectionLock *pProtect   = 0;

/*===========================================================================*\
\*===========================================================================*/

tBool InitPLCIList (void) {
    dhead ("InitPLCIList", DEBUG_PLCIList);
    dassert (pProtect == 0);
    dassert (pPLCIList == 0);
    pProtect  = new CProtectionLock;
    pPLCIList = new CSortPointerList;
    if (pProtect && pPLCIList) {
        return vTrue;
    }
    delete pProtect;
    delete pPLCIList;
    pProtect  = 0;
    pPLCIList = 0;
    return vFalse;
}

/*===========================================================================*\
\*===========================================================================*/

void DeinitPLCIList (void) {
    dhead ("DeinitPLCIList", DEBUG_PLCIList);
    dassert (pProtect != 0);
    dassert (pPLCIList != 0);
    dassert (pPLCIList->IsEmpty() == vTrue);
    delete pProtect;
    delete pPLCIList;
    pProtect  = 0;
    pPLCIList = 0;
}

/*===========================================================================*\
\*===========================================================================*/

tBool TestAndSetPLCIEntry (tUInt PLCI) {
    dhead ("TestAndSetPLCI", DEBUG_PLCIList);
    dassert (PLCI > 0);
    dassert (PLCI <= 0xFFFF);
    dwarning (pPLCIList != 0);
    tBool fret = vFalse;
    if (pPLCIList != 0) {
        tUByte cntrl    = (tUByte)(PLCI & 0x7F);
        tUByte fieldPos = (tUByte)((PLCI >> 11) & 0x1F);
        tUByte fieldBit = (tUByte)(1 << ((PLCI >> 8) & 0x7));
        dassert (cntrl != 0);
        dassert (fieldBit != 0);
        dassert (pProtect != 0);
        pProtect->Begin();
        CPLCIListerElement *pElement = (CPLCIListerElement *)pPLCIList->Find ((void *)cntrl);
        if (pElement == 0) {
            pElement = new CPLCIListerElement;
            if (pElement) {
                pElement->Counter             = 1;
                pElement->Controller          = cntrl;
                pElement->PLCIField[fieldPos] = fieldBit;
                pPLCIList->Add (pElement);
                fret = vTrue;
            }
        } else {
            if (!(pElement->PLCIField[fieldPos] & fieldBit)) {
                pElement->Counter++;
                pElement->PLCIField[fieldPos] |= fieldBit;
                fret = vTrue;
            }
        }
        pProtect->End();
    }
    RETURN ('x', fret);
}

/*===========================================================================*\
\*===========================================================================*/

void UnsetPLCIEntry (tUInt PLCI) {
    dhead ("UnsetPLCIEntry", DEBUG_PLCIList);
    dassert (PLCI > 0);
    dassert (PLCI <= 0xFFFF);
    tUByte fieldPos = (tUByte)((PLCI >> 11) & 0x1F);
    tUByte fieldBit = (tUByte)(1 << ((PLCI >> 8) & 0x7));
    dassert ((PLCI & 0x7F) != 0);
    dassert (fieldBit != 0);
    dassert (pProtect != 0);
    dassert (pPLCIList != 0);
    pProtect->Begin();
    CPLCIListerElement *pElement = (CPLCIListerElement *)pPLCIList->Find ((void *)(tUByte)(PLCI & 0x7F));
    if (pElement && (pElement->PLCIField[fieldPos] & fieldBit)) {
        dassert (pElement->Counter > 0);
        pElement->Counter--;
        if (pElement->Counter == 0) {
            delete pElement;
        } else {
            pElement->PLCIField[fieldPos] &= ~fieldBit;
        }
        pProtect->End();
    }
}

/*===========================================================================*\
\*===========================================================================*/
