/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include <sys/file.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <errno.h>
#include "faxsend.h"
#include "atomar.h"
#include "osImport.h"
#include "ConfAssi.h"
#include "FilePars.h"

/*---------------------------------------------------------------------------*\
    from capiutils.h:
\*---------------------------------------------------------------------------*/

extern "C" char *capi_info2str(c_word reason);

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define FAX_FILTER_BUFFER_SIZE       1000
#define CAPI_BUFFER_SIZE             CAPI_MAX_DATA_SIZE

/*===========================================================================*\
\*===========================================================================*/

CFaxSend::CFaxSend (void (*FinishedFunc)(void)) {
    dhead ("CFaxSend - Constructor", DEBUG_CFaxSend);
    finishFunc    = FinishedFunc;
    VResolution   = 0;
    FaxFileD      = -1;
    hLogFile      = 0;
    hJobFile      = 0;
    enableEcho    = vFalse;
    SendDataInUse = 0;
    exitState     = 0;
}

/*===========================================================================*\
\*===========================================================================*/

CFaxSend::~CFaxSend (void) {
    dhead ("CFaxSend - Destructor", DEBUG_CFaxSend);
    if (FaxFileD >= 0) {
        close (FaxFileD);
        FaxFileD = -1;
    }
    if (hJobFile) {
        fclose (hJobFile);
        hJobFile = 0;
    }
    if (hLogFile != 0) {
        fclose (hLogFile);
        hLogFile = 0;
    }
}

/*===========================================================================*\
\*===========================================================================*/

tBool CFaxSend::Config (char *confName, char *devName, FaxFormatType faxType, int argc, char **argv) {
    dhead ("CFaxSend::Config", DEBUG_CFaxSend);
    dparams ("%s,%x,%x", confName, faxType, argc);
    dassert (confName != 0);
    dassert (devName != 0);
    dassert (argv != 0);
    CDynamicString DeviceName;
    DeviceName.Set (devName);
    format = faxType;
    optind = 1;
    CConfigFileParser FileParser;
    switch (FileParser.ParseFile ((tUChar *)confName)) {
    case CConfigFileParser ::i_Err_OpenFailed:
        printf ("Cant open config file \"%s\". Use default values.\n", confName);
        break;
    case CConfigGrammarParser::i_Okay:
        break;
    case CConfigFileParser ::i_Err_MemoryFull:
        dassert (0);
        printf ("Running out of memory during reading of config file!\n");
        return vFalse;
    default:
        dassert (0);
        printf ("Cant read config file \"%s\"!\n", confName);
        return vFalse;
    }
    ConfigAssignment_Item MainAssignments[] = {
        { "LogFile",        ConfigAssignment_String, &LogFileName },
        { "LogFileMode",    ConfigAssignment_UInt,   &LogFileMode },
        { "LogTraceLevel",  ConfigAssignment_UInt,   &LogLevel },
        { 0,                ConfigAssignment_UInt,   0 }
    };
    ConfigAssignment_Item Assignments[] = {
        { "HylafaxDeviceName",  ConfigAssignment_String, &DeviceName },
        { "FAXNumber",          ConfigAssignment_String, &FaxNumber },
        { "LocalIdentifier",    ConfigAssignment_String, &Identifier },
        { "OutgoingMSN",        ConfigAssignment_String, &OutMSN },
        { "NumberPrefix",       ConfigAssignment_String, &NumberPrefix },
        { "OutgoingController", ConfigAssignment_UInt,   &OutController },
        { "SuppressMSN",        ConfigAssignment_UInt,   &SuppressMSN },
        { "UseISDNFaxService",  ConfigAssignment_UInt,   &UseFaxCIPValue },
        { 0,                    ConfigAssignment_UInt,   0 }
    };
    CConfigParserSection *mainSect = FileParser.GetMainSection();
    if (mainSect) {
        ConfigValuesAssignment (&mainSect->ValueList, &MainAssignments[0]);
        for (CConfigParserSection *laufSect = (CConfigParserSection *)mainSect->SectionList.GetFirst();
             laufSect; laufSect = (CConfigParserSection *)laufSect->GetNext()) {

            if (ConfigValuesTestName (&laufSect->ValueList, &Assignments[0]) != 1) {
                ConfigValuesAssignment (&laufSect->ValueList, &Assignments[0]);
            }
        }
        mainSect->ValueList.RemoveAndDeleteAll();
        mainSect->SectionList.RemoveAndDeleteAll();
    }

    char ch;
    while ((ch = getopt (argc, argv, "Hh?vlpnf:C:c:d:m:")) != -1) {
        switch (ch) {
        case 'c':
            if (mainSect && optarg && *optarg) {
                CDynamicString value;
                value.Set (optarg);
                if (value.IsEmpty() == vFalse) {
                    FileParser.ParseLine (&value);
                    ConfigValuesAssignment (&mainSect->ValueList, &MainAssignments[0]);
                    ConfigValuesAssignment (&mainSect->ValueList, &Assignments[0]);
                    mainSect->ValueList.RemoveAndDeleteAll();
                }
            }
            break;
        case 'v':
            enableEcho = vTrue;
            break;
        }
    }
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

CFaxSend::ctInfo CFaxSend::Send (tUChar *TelNumber, CConfigParserString *FaxFileNames, tUInt Resolution,
                                 tUInt jobID, tUInt *pCommID) {
    dhead ("CFaxSend::Send", DEBUG_CFaxSend);
    dparams ("%s,%s,%u,%u", TelNumber, FaxFileNames->GetPointer(), Resolution, jobID);
    dassert (TelNumber != 0);
    dassert (FaxFileNames != 0);
    dassert (FaxFileNames->IsEmpty() == vFalse);
    dassert (pCommID != 0);
    if (  (LogFileName.IsEmpty() == vFalse) && LogLevel) {
        hLogFile = fopen ((char *)LogFileName.GetPointer(), "a");
        dprint ("LogFile=%x-%s ", hLogFile, (char *)LogFileName.GetPointer());
        if (hLogFile) {
            fchmod (fileno (hLogFile), LogLevel);
        }
    }
    char *ModeText;
    switch (format) {
    default:
        ModeText = "Hylafax";
        *pCommID = GetJobFile();
        if (hJobFile) {
            WriteLog (LOG_INFO, "SESSION BEGIN %08u %s\n", jobID, FaxNumber.GetPointer());
            WriteLog (LOG_INFO, "SEND FAX: JOB %u DEST %s COMMID %08u\n", jobID, TelNumber, *pCommID);
        }
        break;
    case FaxFormat_TIFF:
        ModeText = "TIFF";
        break;
    case FaxFormat_G3:
        ModeText = "G3";
        break;
    case FaxFormat_SFF:
        ModeText = "SFF";
        break;
    }
    CDynamicString UseTelNumber;
    UseTelNumber.Set (&NumberPrefix);
    UseTelNumber.Append (TelNumber);
    if (UseTelNumber.DetectAnError() == vTrue) {
        WriteLog (LOG_ERR, "Memory Error during initialisation.\n");
        return i_MemoryError;
    }

    WriteLog (LOG_INFO, "Try to send a fax to number %s in %s mode.\n", UseTelNumber.GetPointer(), ModeText);
    VResolution = Resolution;
    switch (format) {
    case FaxFormat_Hylafax:
    case FaxFormat_TIFF:
        if (TifFilter.Open (FaxFileNames->GetPointer()) == vFalse) {
            WriteLog (LOG_ERR, "Cant open TIF-File %s!\n", FaxFileNames->GetPointer());
            exitState = 1;
            return i_FileNotFound;
        }
        Resolution = (VResolution) ? VResolution : (TifFilter.GetYRes()) ? TifFilter.GetYRes() : 196;
        if (SffFilter.Open ((Resolution >= 196) ? 1 : 0, 0) == vFalse) {
            WriteLog (LOG_ERR, "Cant allocate memory for data conversion to the SFF format!\n");
            exitState = 11;
            return i_FilterError;
        }
        ModeText = "TIFF";
        break;
    case FaxFormat_G3:
        if (!VResolution) {
            VResolution = Resolution = 196;
        }
        if (SffFilter.Open ((Resolution >= 196) ? 1 : 0, 0) == vFalse) {
            WriteLog (LOG_ERR, "Cant allocate memory for data conversion to the SFF format!\n");
            exitState = 11;
            return i_FilterError;
        }
        // no break!
    case FaxFormat_SFF:
        FaxFileD = open ((char *)FaxFileNames->GetPointer(), O_RDONLY);
        if (FaxFileD < 0) {
            WriteLog (LOG_ERR, "Cant open %s-File %s!\n", ModeText, FaxFileNames->GetPointer());
            SffFilter.Close();
            exitState = 1;
            return i_FileNotFound;
        }
        break;
    }
    WriteLog (LOG_INFO, "Dial and starting transfer of %s-File %s with %s resolution.\n", ModeText,
              FaxFileNames->GetPointer(), (Resolution >= 196) ? "fine" : "normal");
    exitState       = 10;
    FaxFileList     = (CConfigParserString *)FaxFileNames->GetNext();
    finishedFaxFile = vFalse;
    SendDataInUse   = 0;

    cu_B3Configuration B3Config;
    memset (&B3Config, '\0', sizeof (B3Config));
    B3Config.Options    = (Resolution > 98) ? 1 : 0;
    B3Config.Format     = 0;
    B3Config.StationID  = FaxNumber.GetPointer();
    B3Config.HeadLine   = Identifier.GetPointer();

    SetB1Parameters (b1prot_G3Fax);
    SetB2Parameters (b2prot_G3Fax);
    SetB3Parameters (b3prot_G3Fax_extended, &B3Config);
    SetNCPI (b3prot_G3Fax, 0);
    SetOwnNumber (OutMSN.GetPointer(), (SuppressMSN) ? CALLNUM_INDICATOR_SUPPRESS : CALLNUM_INDICATOR_DEFAULT);

    MakeCall (OutController, UseTelNumber.GetPointer(), (UseFaxCIPValue) ? cipvalue_G3Fax : cipvalue_3_1kHzAudio);
    return i_Okay;
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::IncomingCall (tUInt, c_cipvalue, tUChar *, tUChar *) {
    dhead ("CFaxSend::IncomingCall", DEBUG_CFaxSend);
    // Nothing to do!
    dwarning (0);
    AnswerCall (vFalse);
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::DoConnecting (void) {
    dhead ("CFaxSend::DoConnecting", DEBUG_CFaxSend);
    // Nothing to do!
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::ConnectProceeding (void) {
    dhead ("CFaxSend::ConnectProceeding", DEBUG_CFaxSend);
    // Nothing to do!
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::IsConnected (cu_NCPI *pNCPI) {
    dhead ("CFaxSend::IsConnected", DEBUG_CFaxSend);
    // TODO: examine NCPI
    WriteLog (LOG_INFO, "Connection established.\n");
    if (pNCPI) {
        if (pNCPI->ReceiveID && *pNCPI->ReceiveID) {
            WriteLog (LOG_INFO, "\tStationID = %s\n" , pNCPI->ReceiveID);
        }
        if (pNCPI->Rate) {
            WriteLog (LOG_INFO, "\tBaudRate  = %u\n", pNCPI->Rate);
        }
        char szFlags[MAX_STRING_SIZE] = "";
        if (pNCPI->Options & 0x0001) {
            strcat (szFlags, "HighRes, ");
            UseOnlyLowRes = vFalse;
        } else {
            UseOnlyLowRes = vTrue;
        }
        if (pNCPI->Options & 0x0002) {
            strcat (szFlags, "Polling, ");
        }
        if (pNCPI->Options & 0x0004) {
            strcat (szFlags, "PollAfter, ");
        }
        if (pNCPI->Options & 0x8000) {
            strcat (szFlags, "No_ECM, ");
        }
        if (*szFlags) {
            WriteLog (LOG_INFO, "\tFlags     = %s\n", szFlags);
        }
    }
    exitState = 11;
    SendData();
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::PutDataComplete (void *hDataID, c_info) {
    dhead ("CFaxSend::PutDataComplete", DEBUG_CFaxSend);
    // TODO: error handling
    delete [] (tUChar *)hDataID;
    SendData();
    if (IsPutQueueEmpty() == vTrue) {
        Disconnect();
    }
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::GetData (tUByte *, tUShort, tUShort DataHandle) {
    dhead ("CFaxSend::GetData", DEBUG_CFaxSend);
    // Nothing to do!
    dwarning (0);
    GetDataComplete (DataHandle);
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::DoDisconnecting (c_info, cu_NCPI *) {
    dhead ("CFaxSend::DoDisconnecting", DEBUG_CFaxSend);
    if (finishedFaxFile == vFalse) {
        WriteLog (LOG_INFO, "Connection was dropped before Fax was completely transfered");
        exitState = 12;
    }
    finishedFaxFile = vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::IsDisconnected (c_info Reason) {
    dhead ("CFaxSend::IsDisconnected", DEBUG_CFaxSend);
    WriteLog (LOG_INFO, "Connection dropped with Reason 0x%X (%s).\n", Reason, capi_info2str (Reason));
    // connection dropped before established (10) or transfer sill in progress (11)
    if ((exitState == 10) || (exitState == 11)) {
        exitState = (exitState == 11) ? 12 : 10;
        switch ((Reason >> 8) & 0xFF) {
        case 0x00:
        case 0x10:
            exitState = 2;
            break;
        case 0x11:
        case 0x20:
        case 0x30:
            exitState = 3;
            break;
        case 0x33:
            exitState = ((Reason & 0xFF) == 0x01) ? 5 : 10;
            break;
        case 0x34:
            switch (Reason & 0xFF) {
            case 0x91:
            case 0xA2:
            case 0xA9:
                exitState = 4;
                break;
            }
            break;
        }
    }
    finishedFaxFile = vTrue;
    SffFilter.Close();
    if (FaxFileD >= 0) {
        close (FaxFileD);
        FaxFileD = -1;
    }
    if (hJobFile) {
        WriteLog (LOG_INFO, "SESSION END\n");
        fclose (hJobFile);
        hJobFile = 0;
    }
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::IsReleased (c_info /*Reason*/) {
    dhead ("CFaxSend::IsReleased", DEBUG_CFaxSend);
    if (hLogFile != 0) {
        fclose (hLogFile);
        hLogFile = 0;
    }
    if (finishFunc) {
        finishFunc();
    }
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::SendData (void) {
    dhead ("CFaxSend::SendData", DEBUG_CFaxSend);
    tUInt   len    = 0;
    tUChar *buffer = 0;
    if (atomicExchange (&SendDataInUse, 1) == 0) {
        while ((IsPutQueueFull() == vFalse) && (finishedFaxFile == vFalse)) {
            buffer = new tUChar [CAPI_BUFFER_SIZE];
            if (!buffer) {
                WriteLog (LOG_WARNING, "Allocating memory for new buffer failed!\n");
                dwarning (0);
                break;
            }
            if (format == FaxFormat_SFF) {
                len = read (FaxFileD, buffer, CAPI_BUFFER_SIZE);
                switch (len) {
                case 0:
                    WriteLog (LOG_INFO, "Fax completely transfered.\n");
                    finishedFaxFile = vTrue;
                    exitState = 0;
                    break;
                case -1:
                    WriteLog (LOG_ERR, "Failed to read data from file!\n");
                    // TODO: handle error
                    len = 0;
                    break;
                }
            } else {
                FillSffFilter();
                if (SffFilter.CurSize()) {
                    len = SffFilter.Get (buffer, CAPI_BUFFER_SIZE);
                }
            }
            if (len) {
                if (UseOnlyLowRes == vTrue) {
                    //TODO: eliminate 2 line on fine faxes
                    //SffLowResFilter.Change (buffer, len);
                }
                PutData (buffer, len, buffer);
                buffer = 0;
                len    = 0;
            }
            delete [] buffer;
        }
    }
    atomicSet (&SendDataInUse, 0);
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::FillSffFilter (void) {
    dhead ("CFaxSend::FillSffFilter", DEBUG_CFaxSend);
    CSFilter_fTif2Huf::ctInfo info;
    tUChar                    buffer[FAX_FILTER_BUFFER_SIZE];
    tUInt                     len;
    if (finishedFaxFile == vTrue) {
        return;
    }
    while (SffFilter.CurSize() < CAPI_BUFFER_SIZE) {
        if (format == FaxFormat_G3) {
            len  = read (FaxFileD, buffer, FAX_FILTER_BUFFER_SIZE);
            switch (len) {
            case 0:
                info = CSFilter_fTif2Huf::i_LastPageEnd;
                break;
            case -1:
                len = 0;
                WriteLog (LOG_ERR, "Failed to read data from file!\n");
                // TODO: Handle errors
                // no break!
            default:
                info = CSFilter_fTif2Huf::i_Okay;
                break;
            }
        } else {
            len = TifFilter.GetToPageEnd (buffer, FAX_FILTER_BUFFER_SIZE, &info);
        }
        if (len) {
            for (tUInt i = 0; i < len; i++) {
                SffFilter.PutChar (buffer[i]);
            }
            switch (info) {
            case CSFilter_fTif2Huf::i_PageEnd:
                SffFilter.PutPageHeader (vFalse);
                break;
            case CSFilter_fTif2Huf::i_LastPageEnd: {
                tBool   isFinish = vTrue;
                TifFilter.Close();
                WriteLog (LOG_INFO, "Fax file completely transfered.\n");
                for (; (FaxFileList); FaxFileList = (CConfigParserString *)FaxFileList->GetNext()) {
                    dassert (FaxFileList->IsEmpty() == vFalse);
                    dprint ("FileName=%s ", FaxFileList->GetPointer());
                    if (format == FaxFormat_G3) {
                        close (FaxFileD);
                        FaxFileD = open ((char *)FaxFileList->GetPointer(), O_RDONLY);
                        if (FaxFileD >= 0) {
                            WriteLog (LOG_INFO, "Start transfer of G3-File %s.\n", FaxFileList->GetPointer());
                            FaxFileList = (CConfigParserString *)FaxFileList->GetNext();
                            isFinish    = vFalse;
                            break;
                        }
                        WriteLog (LOG_ERR, "Cant open next G3-File %s!\n", FaxFileList->GetPointer());
                    } else if (TifFilter.Open (FaxFileList->GetPointer()) == vTrue) {

                        tUInt Resolution = (VResolution) ? VResolution :
                                           (TifFilter.GetYRes()) ? TifFilter.GetYRes() : 196;
                        SffFilter.SetVRes ((Resolution >= 196) ? 1 : 0);
                        WriteLog (LOG_INFO, "Start transfer of TIFF-File %s with %s resolution.\n",
                                  FaxFileList->GetPointer(), (Resolution >= 196) ? "fine" : "normal");
                        FaxFileList = (CConfigParserString *)FaxFileList->GetNext();
                        isFinish    = vFalse;
                        break;
                    } else {
                        WriteLog (LOG_ERR, "Cant open TIFF-File %s.\n", FaxFileList->GetPointer());
                    }
                }
                finishedFaxFile = isFinish;
                SffFilter.PutPageHeader (isFinish);
                if (finishedFaxFile == vTrue) {
                    exitState = 0;
                }
        dprint ("i_LastPageEnd-Recv (exitState=%x isFinish=%x) ", exitState, isFinish);
                return; }
            case CSFilter_fTif2Huf::i_Okay:
                break;
            default:
                // TODO: handle errors!
                return;
            }
        }
    }
}

/*===========================================================================*\
\*===========================================================================*/

void CFaxSend::WriteLog (tSInt priority, char *text, ...) {
    dhead ("CFaxSend::WriteLog", DEBUG_CFaxSend);
    char   *statusText = 0;
    char    timebuf[MAX_STRING_SIZE];
    timeval tv;
    va_list params;
    va_start (params, text);

    gettimeofday (&tv, 0);
    strftime (timebuf, MAX_STRING_SIZE, "%h %d %T", localtime((time_t*) &tv.tv_sec));
    sprintf (timebuf + strlen(timebuf), ".%02lu: [%5d]: ", tv.tv_usec / 10000, getpid());

    if (hJobFile) {
        fprintf (hJobFile, timebuf);
        vfprintf (hJobFile, text, params);
    }
    if (enableEcho) {
        vprintf (text, params);
    }

    switch (priority) {
    default:
        if (LogLevel < LOG_TRACE_ALL) {
            return;
        }
        break;
    case LOG_INFO:
    case LOG_NOTICE:
        if (LogLevel < LOG_TRACE_INFOS) {
            return;
        }
        statusText = "INFO: ";
        break;
    case LOG_WARNING:
        if (LogLevel < LOG_TRACE_WARNINGS) {
            return;
        }
        statusText = "WARNING: ";
        break;
    case LOG_ERR:
        if (LogLevel < LOG_TRACE_ERRORS) {
            return;
        }
        statusText = "ERROR: ";
        break;
    }

    if (hLogFile) {
        fprintf (hLogFile, timebuf);
        fprintf (hLogFile, DEFAULT_SENDPROG_NAME " - ");
        if (statusText) {
            fprintf (hLogFile, statusText);
        }
        vfprintf (hLogFile, text, params);
    }
    if (priority == LOG_ERR) {
        char errortext[MAX_STRING_SIZE];
        vsnprintf (errortext, MAX_STRING_SIZE, text, params);
        openlog (DEFAULT_SENDPROG_NAME, LOG_PERROR, 0);
        syslog (priority, errortext);
        closelog();
    }
    va_end (params);
}

/*===========================================================================*\
\*===========================================================================*/

tUInt CFaxSend::GetJobFile (void) {
    dhead ("CFaxSend::GetJobFile", DEBUG_CFaxSend);
    dassert (format == FaxFormat_Hylafax);
    if (hJobFile != 0) {
        fclose (hJobFile);
        hJobFile = 0;
    }
    char  astring[MAX_STRING_SIZE];
    tSInt len;
    tUInt tmpJobFileNr = 1;
    tSInt tmpFileD     = -1;
    tSInt SeqFile      = open (HYLAFAX_LOG_DIR HYLAFAX_SEQF_FILE, O_CREAT | O_RDWR, 0644);
    if (SeqFile >= 0) {
        flock (SeqFile, LOCK_EX);
        len = read (SeqFile, astring, MAX_STRING_SIZE);
        astring[(len <= 0) ? 0 : len] = '\0';
        tmpJobFileNr = atoi (astring) + 1;
    }
    for (tUInt trys = MAX_TRY_COUNT; trys > 0; trys--, tmpJobFileNr++) {
        if (tmpJobFileNr < 1 || tmpJobFileNr > MAX_SEQ_NUMBER) {
            tmpJobFileNr = 1;
        }
        sprintf (astring, HYLAFAX_LOG_DIR "c%08u", tmpJobFileNr);
        tmpFileD = open (astring, O_RDWR | O_CREAT | O_EXCL, LogFileMode);
        dprint ("try(%s,%u,%d) ", astring, strlen (astring), tmpFileD);
        if (tmpFileD >= 0) {
            flock (tmpFileD, LOCK_EX | LOCK_NB);
            hJobFile = fdopen (tmpFileD, "w");
            dprint ("fdopen(%x,%d) ", hJobFile, tmpFileD);
            if (hJobFile != 0) {
                if (SeqFile >= 0) {
                    sprintf (astring , "%u", tmpJobFileNr);
                    len = strlen (astring);
                    lseek (SeqFile, 0, SEEK_SET);
                    write (SeqFile, astring, len);
                    ftruncate (SeqFile, len);
                }
                break;
            }
            close (tmpFileD);
            tmpFileD = -1;
            continue;
        }
        if (errno != EEXIST) {
            break;
        }
    }
    if (SeqFile >= 0) {
        close (SeqFile);
    }
    return (hJobFile) ? tmpJobFileNr : 0;
}

/*===========================================================================*\
\*===========================================================================*/


