/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include <stdio.h>
#include <semaphore.h>
#include <unistd.h>
#include "faxsend.h"
#include "ConfAssi.h"
#include "FilePars.h"
#include "DynQueue.h"
#include "DbgFile.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

static sem_t syncsema;

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

void FinishFunc (void) {
    sem_post (&syncsema);
}

/*===========================================================================*\
\*===========================================================================*/

void WriteQFile (char *QFileName, tUInt commID, tUInt exitState) {
    dhead ("WriteQFile", DEBUG_CFaxSend);
    dassert (QFileName != 0);
    CDynamicQueue textQ (60, sizeof (char *));
    char  text[4 * MAX_STRING_SIZE];
    char *ne;
    char *statustext = "";
    FILE *qf = fopen (QFileName, "r+");
    if (!qf) {
        return;
    }
    switch (exitState) {
    case 0:
        break;
    case 1:
        statustext = "Cant open fax file.";
        break;
    case 2:
        statustext = "CAPI not installed or active.";
        break;
    case 3:
        statustext = "Error in communication to CAPI.";
        break;
    case 4:
        statustext = "The telephone number is busy or temporary not available.";
        break;
    case 5:
        statustext = "Protocol error layer 1: Mostly your ISDN cable is not inserted.";
        break;
    case 6:
        statustext = "Can't build up B-Channel.";
        break;
    case 7:
        statustext = "Connection build up, but data transfer wasn't possible.";
        break;
    case 10:
        statustext = "CAPI protocol error.";
        break;
    case 11:
        statustext = "Cant initialize conversion filters.";
        break;
    case 12:
        statustext = "Error occured during the fax transfer.";
        break;
    case 13:
        statustext = "Wrong command line or help parameter was specified.";
        break;
    default:
    case 15:
        statustext = "Very hard error.";
        break;
    }

    // we asume that we can always get a whole line with fgets
    while (!feof (qf) && fgets (text, sizeof (text), qf)) {
        if (!*text) {
            continue;
        }
        ne = strchr (text, ':');
        if (!ne || (ne == text)) {
            continue;
        }
        *ne = '\0';
        if (strcasecmp ("status", text) == 0) {
            dprint ("found status ");
            sprintf (ne + 1, "%s\n", statustext);
        } else if (commID && (strcasecmp ("commid", text) == 0)) {
            dprint ("found commid ");
            sprintf (ne + 1, "%08u\n", commID);
        }
        *ne = ':';
        ne = new char[strlen (text) + 1];
        if (ne) {
            strcpy (ne, text);
            if (textQ.Put (&ne) == vFalse) {
                delete [] ne;
            }
        }
    }
    fseek (qf, 0, SEEK_SET);
    ftruncate (fileno (qf), 0);
    while (textQ.Get (&ne) == vTrue) {
        dassert (ne != 0);
        fprintf (qf, ne);
        delete [] ne;
    }
}

/*===========================================================================*\
\*===========================================================================*/

void usage (char *prgName) {
    char *name = strrchr (prgName, '/');
    if (!name) {
        name = prgName;
    } else {
        name++;
    }
    printf ("usage: %s [-f format] [-C cfile] [-c param] [-m dev] [-d number] [-v] [-V] file(s)\n", name);
    printf ("\tformat:  Specify the output format. Allowed types are:\n");
    printf ("\t         HYLAFAX (default), TIFF, SFF, G3.\n");
    printf ("\tcfile:   Name and path of the config file\n");
    printf ("\tparam:   One Entry from the config-file of type: {value}:{data}\n");
    printf ("\tdev:     Virtual device name only needed for Hylafax. Default: faxCAPI\n");
    printf ("\tfile(s): Depending on the format(-f) switch.\n");
    printf ("\t         HYLAFAX: one         qfile\n");
    printf ("\t         SFF:     one         sff file\n");
    printf ("\t         TIFF:    one or more tiff file(s)\n");
    printf ("\t-v       Start in verbose mode\n");
    printf ("\t-V       Show version information\n");
}

/*===========================================================================*\
\*===========================================================================*/
#include "aStdLib.h"

int main (int argc, char **argv) {
    DebugFileOpen();
    dhead ("CFaxSend-main", DEBUG_CFaxSend);
    CFaxSend fs (FinishFunc);
    if (argc < 2) {
        usage (argv[0]);
        return 13;
    }
    CDynamicString TelNumber;
    CPointerList   FaxFileList;
    tUInt          Resolution = 0;
    tUInt          JobID      = 0;

    // find device name
    FaxFormatType format = FaxFormat_Hylafax;
    char  HylafaxConfigName[MAX_STRING_SIZE];
    char *configName = 0;
    char *deviceName = DEFAULT_DEVICE_NAME;
    tUInt commID     = 0;
    int   ch;
    while ((ch = getopt (argc, argv, "Hh?Vvlpnf:C:c:d:m:")) != -1) {
        switch (ch) {
        default:
            usage (argv[0]);
            return 13;
        case 'V':
            printf ("c2faxsend Version: %s (c) AVM GmbH\n", VERSION);
            return 13;
        case 'l':
        case 'p':
        case 'v':
        case 'c':
            break;
        case 'n':
            Resolution = 98;
            break;
        case 'C':
            if (optarg && *optarg) {
                configName = optarg;
            }
            break;
        case 'd':
            if (optarg && *optarg) {
                 TelNumber.Set (optarg);
            }
            break;
        case 'm':
            if (optarg && *optarg) {
                deviceName = optarg;
            }
            break;
        case 'f':
            if (!optarg || !*optarg) {
                break;
            }
            if (strcasecmp (optarg, FORMAT_STRING_HYLAFAX) == 0) {
                format = FaxFormat_Hylafax;
            } else if (strcasecmp (optarg, FORMAT_STRING_SFF) == 0) {
                format = FaxFormat_SFF;
            } else if (strcasecmp (optarg, FORMAT_STRING_TIFF) == 0) {
                format = FaxFormat_TIFF;
            } else if (strcasecmp (optarg, FORMAT_STRING_G3) == 0) {
                format = FaxFormat_G3;
            } else {
                printf ("Wrong format param %s!\n\n", optarg);
                return 13;
            }
            break;
        }
    }
    if (!configName) {
        if (format == FaxFormat_Hylafax) {
            sprintf (HylafaxConfigName, HYLAFAX_CONFIG_PREFIX "%s", deviceName);
            configName = HylafaxConfigName;
        } else {
            configName = DEFAULT_CONFIG_FILE;
        }
    }
    if ((optind + 1 >= argc) && (fs.Config (configName, deviceName, format, argc, argv) == vFalse)) {
        return 1;
    }
    switch (format) {
    case FaxFormat_Hylafax:
        {
            // ignore -d and -n switch
            TelNumber.RemoveAll();
            Resolution = 0;

            CConfigFileParser FileParser;
            switch (FileParser.ParseFile ((tUChar *)argv[optind])) {
            case CConfigFileParser ::i_Err_OpenFailed:
                printf ("Cant open qfile \"%s\".\n", argv[optind]);
                return 3;
                break;
            case CConfigGrammarParser::i_Okay:
                break;
            case CConfigFileParser ::i_Err_MemoryFull:
                dassert (0);
                printf ("Running out of memory during reading of qfile!\n");
                return 3;
                return vFalse;
            default:
                dassert (0);
                printf ("Cant read config file \"%s\"!\n", argv[optind]);
                return 3;
            }
            CConfigParserSection *mainSect = FileParser.GetMainSection();
            if (!mainSect) {
                printf ("Cant find needed information in qfile \"%s\".\n", argv[optind]);
                return 3;
            }
            ConfigAssignment_Item Assignments[] = {
                { "number",     ConfigAssignment_String,     &TelNumber },
                { "fax",        ConfigAssignment_StringList, &FaxFileList },
                { "resolution", ConfigAssignment_UInt,       &Resolution },
                { "jobid",      ConfigAssignment_UInt,       &JobID },
                { 0,            ConfigAssignment_UInt,       0 }
            };
            ConfigValuesAssignment (&mainSect->ValueList, &Assignments[0]);

            // remove numbers from fax list
            CConfigParserString *pLauf = (CConfigParserString *)FaxFileList.GetFirst();
            while (pLauf != 0) {
                dassert (pLauf->GetLen() > 0);
                switch (pLauf->GetChar (0)) {
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                    {
                        CConfigParserString *toDel = pLauf;
                        pLauf = (CConfigParserString *)pLauf->GetNext();
                        delete toDel;
                    }
                    break;
                default:
                    pLauf = (CConfigParserString *)pLauf->GetNext();
                    break;
                }
            }
        }
        break;
    case FaxFormat_SFF:
        {
            CConfigParserString *pNewStr = new CConfigParserString;
            if (pNewStr) {
                if (pNewStr->Set (argv[optind]) == vTrue) {
                    FaxFileList.AddLast (pNewStr);
                } else {
                    delete pNewStr;
                }
            }
        }
        break;
    case FaxFormat_TIFF:
    case FaxFormat_G3:
        {
            CConfigParserString *pNewStr;
            for (; optind < argc; optind++) {
                pNewStr = new CConfigParserString;
                if (pNewStr) {
                    if (pNewStr->Set (argv[optind]) == vTrue) {
                        FaxFileList.AddLast (pNewStr);
                    } else {
                        delete pNewStr;
                    }
                }
            }
        }
        break;
    }
    if (FaxFileList.IsEmpty() == vTrue) {
        printf ("No file to send specified!%s\n\n", (format == FaxFormat_Hylafax) ?
                " Forgot the format option (-f)?" : "");
        usage (argv[0]);
        return 13;
    }
    if ((FaxFileList.IsEmpty() == vTrue) || (TelNumber.IsEmpty() == vTrue)) {
        printf ("No telephone number specified!\n\n");
        usage (argv[0]);
        return 13;
    }
    sem_init (&syncsema, 0, 0);
    if (fs.Send (TelNumber.GetPointer(),
                 (CConfigParserString *)FaxFileList.GetFirst(),
                 Resolution,
                 JobID,
                 &commID) == CFaxSend::i_Okay) {
        sem_wait (&syncsema);
    }
    sem_destroy (&syncsema);
    dprint ("fs.exitState=%x ", fs.exitState);
    if (format == FaxFormat_Hylafax) {
        WriteQFile (argv[optind], commID, fs.exitState);
        // map error codes from mgetty+sendfax to hylafax
        switch (fs.exitState) {
        case 2:
        case 4:
        case 5:
        case 6:
        case 7:
        case 10:
        case 12:
            // retry
            fs.exitState = 0;
            break;
        case 15:
        default:
            // error
            fs.exitState = 1;
            break;
        case 0:
            // okay
            fs.exitState = 2;
            break;
        case 1:
        case 3:
        case 11:
            // reformat
            fs.exitState = 3;
            break;
        }
    }
    dprint ("fs.exitState=%x ", fs.exitState);
    DebugFileClose();
    return fs.exitState;
}

/*===========================================================================*\
\*===========================================================================*/

