/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "CString.h"
#include "aString.h"
#include "osImport.h"

/*===========================================================================*\
\*===========================================================================*/

CDynamicString::CDynamicString (void) {
    dhead ("CDynamicString-Constructor", DEBUG_CDynamicString);
    pntr        = 0;
    curLen      = 0;
    maxSize     = 0;
    detectError = vFalse;
}

/*===========================================================================*\
\*===========================================================================*/

CDynamicString::~CDynamicString (void) {
    dhead ("CDynamicString-Destructor", DEBUG_CDynamicString);
    tString delPntr = pntr;
    pntr            = 0;
    curLen          = 0;
    maxSize         = 0;
    detectError     = vFalse;
    if (delPntr)  {
        delete [] delPntr;
    }
}

/*===========================================================================*\
\*===========================================================================*/

tBool CDynamicString::Change (tString string, tUInt startPos, tUInt len) {
    dhead ("CDynamicString::Change", DEBUG_CDynamicString);
    dassert (string != 0);
    dassert (startPos <= GetLen());
    if (!len) {
        len = s_strlen (string);
    }
    tUInt endPos = len + startPos;
    if (endPos >= GetMaxSize()) {
        if (DynExpand (endPos + 1) == vFalse) {
            return vFalse;
        }
    }
    dassert (endPos < GetMaxSize());
    dassert (pntr != 0);
    s_strncpy (pntr + startPos, string, len);
    if (endPos > GetLen()) {
        pntr[endPos] = '\0';
        curLen       = endPos;
    }
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

tBool CDynamicString::Change (char *string, tUInt startPos, tUInt len) {
    dhead ("CDynamicString::Change", DEBUG_CDynamicString);
    dassert (string != 0);
    dassert (startPos <= GetLen());
    if (!len) {
        len = s_strlen ((tUChar *)string);
    }
    tUInt endPos = len + startPos;
    if (endPos >= GetMaxSize()) {
        if (DynExpand (endPos + 1) == vFalse) {
            return vFalse;
        }
    }
    dassert (endPos < GetMaxSize());
    dassert (pntr != 0);
    s_strncpy (pntr + startPos, (tUChar *)string, len);
    if (endPos > GetLen()) {
        pntr[endPos] = '\0';
        curLen       = endPos;
    }
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

tBool CDynamicString::Insert (tString string, tUInt startPos, tUInt len) {
    dhead ("CDynamicString::Insert", DEBUG_CDynamicString);
    dassert (string != 0);
    dassert (startPos <= GetLen());
    if (!len) {
        len = s_strlen (string);
    }
    tUInt endPos = len + GetLen();
    if (endPos >= GetMaxSize()) {
        if (DynExpand (endPos + 1) == vFalse) {
            return vFalse;
        }
    }
    dassert (endPos < GetMaxSize());
    dassert (pntr != 0);
    if (GetLen() > startPos) {
        s_strmov (pntr + startPos, len, GetLen() - startPos);
    }
    s_strncpy (pntr + startPos, string, len);
    pntr[endPos] = '\0';
    curLen       = endPos;
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/

void CDynamicString::Get (tString string, tUInt startPos, tUInt len) {
    dhead ("CDynamicString::Get", DEBUG_CDynamicString);
    dassert (string != 0);
    dassert (startPos <= GetLen());
    dassert (len <= GetLen());
    dassert (startPos + len <= GetLen());
    s_strncpy (string, pntr + startPos, len);
    string[len] = '\0';
}

/*===========================================================================*\
\*===========================================================================*/

tUInt CDynamicString::FindChar (tStringChar ch, tUInt startpos) {
    dhead ("CDynamicString::FindChar", DEBUG_CDynamicString);
    for (; startpos < GetLen(); startpos++) {
        if (pntr[startpos] == ch) {
            return startpos;
        }
    }
    return MAXVAL_tUInt;
}

/*===========================================================================*\
\*===========================================================================*/

void CDynamicString::Remove (tUInt startPos, tUInt len) {
    dhead ("CDynamicString::Remove", DEBUG_CDynamicString);
    dassert (startPos <= GetLen());
    dassert (len <= GetLen());
    dassert (startPos + len <= GetLen());
    tUInt endPos = startPos + len;
    if (endPos >= GetLen()) {
        SetLen (startPos);
    } else {
        s_strmov (pntr + endPos, -len, GetLen() - endPos + 1);
    }
}

/*===========================================================================*\
\*===========================================================================*/

tBool CDynamicString::Resize (tUInt newMaxSize) {
    dhead ("CDynamicString::Resize", DEBUG_CDynamicString);
    dassert (GetLen() + 1 < newMaxSize);
    tString delPntr = 0;
    tString newPntr = new tStringChar [newMaxSize];
    if (!newPntr) {
        detectError = vTrue;
        return vFalse;
    }
    if (pntr) {
        s_strncpy (newPntr, pntr, GetLen() + 1);
    } else {
        newPntr[0] = '\0';
    }
    delPntr = pntr;
    pntr    = newPntr;
    maxSize = newMaxSize;
    delete [] delPntr;
    return vTrue;
}

/*===========================================================================*\
\*===========================================================================*/
