/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _CONFPARS_CPP_
#define _CONFPARS_CPP_

#include "ConfPars.h"
#include "osImport.h"

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

enum GrammarParseState {
    ps_SearchName,
    ps_SearchAssign,
    ps_SearchData,
    ps_SearchEnd
};

/*===========================================================================*\
\*===========================================================================*/

CConfigGrammarParser::CConfigGrammarParser (CConfigParserSection *pMainSection) {
    dhead ("CConfigGrammarParser-Constructor", DEBUG_ConfigParser);
    if (!pMainSection) {
        pMainSection = new CConfigParserSection;
    }
    pCurSec = pMainSec = pMainSection;
}

/*===========================================================================*\
\*===========================================================================*/

CConfigGrammarParser::~CConfigGrammarParser (void) {
    dhead ("CConfigGrammarParser-Destructor", DEBUG_ConfigParser);
    // TODO?: delete is not always the best idea here
    delete pMainSec;
    pMainSec = 0;
}

/*===========================================================================*\
\*===========================================================================*/

CConfigGrammarParser::ctInfo CConfigGrammarParser::ParseLine (CDynamicString *StrLine) {
    dhead ("CConfigGrammarParser::ParseLine", DEBUG_ConfigParser);
    ctInfo fret = i_Okay;
    if ((StrLine) && (CConfigLineParser::ParseLine (StrLine) == vFalse)) {
        return i_Err_MemoryFull;
    }
    CPointerList *wordList = GetWordList();
    dwarning (wordList != 0);
    if (wordList) {
        fret = ParseWords (wordList);
        if ((fret != i_Pending) || (!StrLine)) {
            wordList->RemoveAndDeleteAll();
        }
    }
    return fret;
}

/*===========================================================================*\
\*===========================================================================*/

CConfigGrammarParser::ctInfo CConfigGrammarParser::ParseWords (CPointerList *WordList) {
    dhead ("CConfigGrammarParser::ParseWordsX", DEBUG_ConfigParser);
    dassert (WordList != 0);
    CConfigParserString *pPStr;
    
    // Test for line concernator '\\'
    pPStr = (CConfigParserString *)WordList->GetLast();
    if (!pPStr) {
        dprint (" WordList is EMPTY ");
        RETURN ('x', i_Okay);
    }
    dassert (pPStr->GetLen() > 0);
    while (pPStr->GetChar (0) == '#') {
        pPStr = (CConfigParserString *)pPStr->GetPrev();
        if (!pPStr) {
            RETURN ('x', i_Okay);
        }
    }
    if ((pPStr->GetLen() == 1) && (pPStr->GetChar (0) == '\\')) {
        // found line concernator and all later comments
        CConfigParserString *delPStr;
        while (pPStr) {
            delPStr = pPStr;
            pPStr   = (CConfigParserString *)pPStr->GetNext();
            delete delPStr;
        }
        RETURN ('x', i_Pending);
    }

    // parse words
    CConfigParserValue *pCurValue  = 0;
    GrammarParseState   ParseState = ps_SearchName;

    // parse current string list
    while ((pPStr = (CConfigParserString *)WordList->GetFirst()) != 0) {
        dassert (pPStr->GetLen() > 0);
        dassert (pCurSec != 0);
        //dprint ("CurWord=%s ", pPStr->GetPointer());
        switch (pPStr->GetChar (0)) {
        case '{': {
            dprint ("Found: Section OPEN\n");
            dassert (pPStr->GetLen() == 1);
            if (pCurValue) {
                if (pCurValue->DataList.IsEmpty() == vFalse) {
                    pCurSec->ValueList.AddLast (pCurValue);
                } else  {
                    delete pCurValue;
                }
                pCurValue = 0;
            }
            CConfigParserSection *newSec = new CConfigParserSection;
            if (!newSec) {
                return i_Err_MemoryFull;
            }
            newSec->pUpperSection = pCurSec;
            pCurSec->SectionList.AddLast (newSec);
            pCurSec = newSec;
            break;
        }
        case '}':
            dprint ("Found: Section CLOSE\n");
            dassert (pPStr->GetLen() == 1);
            if (pCurValue) {
                if (pCurValue->DataList.IsEmpty() == vFalse) {
                    pCurSec->ValueList.AddLast (pCurValue);
                } else  {
                    delete pCurValue;
                }
                pCurValue = 0;
            }
            if (pCurSec->pUpperSection == 0) {
                return i_Err_TwoMuchSectionEnds;
            }
            pCurSec = pCurSec->pUpperSection;
            break;

        case '"':
            // Remove "
            if ((pPStr->GetLen() <= 2) || (pPStr->GetLastChar() != '"')) {
                // do nothing !!!!

                //delete pCurValue;
                //return i_Err_QuotationError;
            } else {
                pPStr->SetLen (-1);
                pPStr->Remove (0, 1);
            }
            // TODO?: Remove \ from \" ?
            // no break!

        default:
            if (!pCurValue) {
                pCurValue = new CConfigParserValue;
                if (!pCurValue) {
                    delete pCurValue;
                    return i_Err_MemoryFull;
                }
                dprint ("Name=%s ", pPStr->GetPointer());
                pCurValue->Name.Set (pPStr);
                ParseState = ps_SearchAssign;
            } else {
                switch (ParseState) {
                case ps_SearchName:
                case ps_SearchAssign:
                case ps_SearchEnd:
                    ParseState = ps_SearchName;
                    delete pCurValue;
                    pCurValue = 0;
                    return i_Err_WrongAssignmentCoding;
                case ps_SearchData:
                    dprint ("Data=%s ", pPStr->GetPointer());
                    pPStr->RemoveFromList();
                    pCurValue ->DataList.AddLast (pPStr);
                    pPStr = 0;
                    break;
                }
            }
            break;

        case '=':
        case ':':
            dassert (pPStr->GetLen() == 1);
            switch (ParseState) {
            case ps_SearchData:
                dwarning (0);
                break;
            case ps_SearchAssign:
                ParseState = ps_SearchData;
                break;
            default:
                delete pCurValue;
                return i_Err_WrongAssignmentCoding;
            }
            break;

        case ',':
            if (ParseState != ps_SearchData) {
                delete pCurValue;
                return i_Err_WrongAssignmentCoding;
            }
            break;

        //case ';':
        //    if (pCurValue) {
        //        if (pCurValue->DataList.IsEmpty() == vFalse) {
        //            pCurSec->ValueList.AddLast (pCurValue);
        //        } else  {
        //            delete pCurValue;
        //        }
        //        pCurValue = 0;
        //    }
        //    ParseState = ps_SearchName;
        //    break;

        case '#':
            break;
        }
        delete pPStr;
    }
    if (pCurValue) {
        if (pCurValue->DataList.IsEmpty() == vFalse) {
            pCurSec->ValueList.AddLast (pCurValue);
        } else  {
            delete pCurValue;
        }
    }
    RETURN ('x', i_Okay);
}


/*===========================================================================*\
\*===========================================================================*/

#endif

