/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _PTRLIST_H_
#define _PTRLIST_H_

#include "aTypes.h"
#include "Protect.h"

/*===========================================================================*\
\*===========================================================================*/

class CPointerListElement {
public:
    CPointerListElement (void);
    virtual ~CPointerListElement (void);

    CPointerListElement *GetNext (void);
    CPointerListElement *GetPrev (void);
    void RemoveFromList (void);
    void AddAfter (CPointerListElement *pElement);
    void AddBefore (CPointerListElement *pElement);

    void SetProtection (CProtectionLock *pProtection);
private:
    CProtectionLock *pProtect;
    CPointerListElement *pNext;
    CPointerListElement *pPrev;
};


/*===========================================================================*\
\*===========================================================================*/

class CPointerList {
public:
    typedef tResult (*ctFindFunc) (CPointerListElement *pElement, void *RefData);

    CPointerList (void);
    virtual ~CPointerList (void);
    tBool IsEmpty (void);
    tUInt CountElements (void);

    CPointerListElement *GetFirst (void);
    CPointerListElement *GetLast (void);
    void AddFirst (CPointerListElement *pElement);
    void AddLast (CPointerListElement *pElement);

    CPointerListElement *RemoveFirst (void);
    CPointerListElement *RemoveLast (void);
    void RemoveAndDeleteAll (void);

    CPointerListElement *Find (ctFindFunc func, void *RefData);

private:
    CProtectionLock Protect;
    CPointerListElement Head;
    CPointerListElement Tail;
};


/*===========================================================================*\
    inline Functions for CPointerListElement
\*===========================================================================*/

#include "osImport.h"

inline CPointerListElement::CPointerListElement (void) {
    dhead ("CPointerListElement-Constructor", DEBUG_CPointerList);
    pProtect = 0;
    pNext    = 0;
    pPrev    = 0;
}

inline CPointerListElement::~CPointerListElement (void) {
    dhead ("CPointerListElement-Destructor", DEBUG_CPointerList);
    RemoveFromList();
    pProtect = 0;
}

inline CPointerListElement *CPointerListElement::GetNext (void) {
    dhead ("CPointerListElement::GetNext", DEBUG_CPointerList);
    dassert (pNext != 0);
    // the last element is for list managing only
    CPointerListElement *fret = pNext;
    RETURN ('x', (fret->pNext) ? fret : 0);
}

inline CPointerListElement *CPointerListElement::GetPrev (void) {
    dhead ("CPointerListElement::GetPrev", DEBUG_CPointerList);
    dassert (pPrev != 0);
    // the first element is for list managing only
    CPointerListElement *fret = pPrev;
    RETURN ('x',(fret->pPrev) ? fret : 0);
}

inline void CPointerListElement::SetProtection (CProtectionLock *pProtection) {
    dhead ("CPointerListElement::SetProtection", DEBUG_CPointerList);
    dassert (pProtection != 0);
    pProtect = pProtection;
}

/*===========================================================================*\
    inline Functions for CPointerList
\*===========================================================================*/

inline CPointerList::CPointerList (void) {
    dhead ("CPointerList-Constructor", DEBUG_CPointerList);
    // initialize the list
    Head.SetProtection (&Protect);
    Head.AddAfter (&Tail);
}

inline CPointerList::~CPointerList (void) {
    dhead ("CPointerList-Destructor", DEBUG_CPointerList);
    RemoveAndDeleteAll();
}

inline tBool CPointerList::IsEmpty (void) {
    dhead ("CPointerList::IsEmpty", DEBUG_CPointerList);
    return (Head.GetNext() == 0) ? vTrue : vFalse;
}

inline CPointerListElement *CPointerList::GetFirst (void) {
    dhead ("CPointerList::GetFirst", DEBUG_CPointerList);
    return Head.GetNext();
}

inline CPointerListElement *CPointerList::GetLast (void) {
    dhead ("CPointerList::GetLast", DEBUG_CPointerList);
    return Tail.GetPrev();
}

inline void CPointerList::AddFirst (CPointerListElement *pElement) {
    dhead ("CPointerList::AddFirst", DEBUG_CPointerList);
    dparams ("%x", pElement);
    Head.AddAfter (pElement);
}

inline void CPointerList::AddLast (CPointerListElement *pElement) {
    dhead ("CPointerList::AddLast", DEBUG_CPointerList);
    dparams ("%x", pElement);
    Tail.AddBefore (pElement);
}

/*===========================================================================*\
\*===========================================================================*/

#endif

