/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _SORTLIST_H_
#define _SORTLIST_H_

#include "PtrList.h"

/*===========================================================================*\
\*===========================================================================*/

class CSortPListElement : protected CPointerListElement {
public:
    CSortPListElement *GetLower (void);
    CSortPListElement *GetHigher (void);
    void RemoveFromList (void);

    virtual tCompare Compare (void *RefCompData) = 0;  // RefCompData is [tCompare] with class own's RefCompData
    virtual void *GetRefCompData (void) = 0;

    friend class CSortPointerList;
};

/*===========================================================================*\
\*===========================================================================*/

class CSortPointerList : protected CPointerList {
public:
    tBool IsEmpty (void);
    CSortPListElement *GetHighest (void);
    CSortPListElement *GetLowest (void);
    void Add (CSortPListElement *pElement);
    CSortPListElement *Find (ctFindFunc func, void *RefData);
    CSortPListElement *Find (void *CompData);
};

/*===========================================================================*\
    inline functions from CSortPListElement
\*===========================================================================*/

inline CSortPListElement *CSortPListElement::GetLower (void) {
    return (CSortPListElement *)CPointerListElement::GetNext();
}

inline CSortPListElement *CSortPListElement::GetHigher (void) {
    return (CSortPListElement *)CPointerListElement::GetPrev();
}

inline void CSortPListElement::RemoveFromList (void) {
    CPointerListElement::RemoveFromList();
}

/*===========================================================================*\
    inline functions from CSortPointerList
\*===========================================================================*/

inline tBool CSortPointerList::IsEmpty (void) {
    return CPointerList::IsEmpty();
}

inline CSortPListElement *CSortPointerList::GetHighest (void) {
    return (CSortPListElement *)CPointerList::GetFirst();
}

inline CSortPListElement *CSortPointerList::GetLowest (void) {
    return (CSortPListElement *)CPointerList::GetLast();
}

inline CSortPListElement *CSortPointerList::Find (ctFindFunc func, void *RefData) {
    return (CSortPListElement *)CPointerList::Find (func, RefData);
}


/*===========================================================================*\
\*===========================================================================*/

#endif

