/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#ifndef _ASTDLIB_H_
#define _ASTDLIB_H_

#include <stdarg.h>
#include "aTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#define a_isalpha(_c)  (((_c & 0x40) == 0x40) && (_c & 0x1F) && ((_c & 0x1F) < 0x1A))

long a_atol (char *string);

long a_strtoln (const char *nptr, const char **endptr, unsigned maxlength, int base);

__inline long a_strtol (const char *nptr, const char **endptr, int base) {
    return a_strtoln (nptr, endptr, (unsigned) -1, base);
}

unsigned a_ltoan (tSInt value, char *string, tUInt maxlen, tSInt radix);

char *aStrUpr (char *string);

/*---------------------------------------------------------------------------*\
    vsnprintf
        Speichert eine formatierte Ausgabe im buffer mit maximal maxcount
        Zeichen (inklusive abschliessende '\0')
        Aufbau wie _vsnprintf (stdio.h) mit Besonderheiten im format:
        %a: Es wird ein "Format-String" eingefgt der wiederum auch "Format
            Specifications" enthalten kann. Ihm folgt auf dem Parameter-Stack
            eine Variable vom Typ va_list, die die in den "Format-String"
            einzufgenden Werte enthlt.
        %b: "Binre Ausgabe". Ist der "char"-Parameter 0 wird FALSE ausgegeben,
            ansonsten TRUE. Wenn nicht genug Platz ist oder die Precision auf 1
            eingestellt wurde verkrzt sich die Ausgabe auf 'T' und 'F'.
        %t: Einschub nach '\n'
        Zur Zeit werden folgende "Format Specificationen nicht untersttzt:
            - Die Prefixe:  F, N, h, I
            - Die Typen:    f, e, E, g, G, n
            - Precision:    .[Zahl], da auch keine Float's untersttzt!
        Der Prefix 'l' bzw. 'L' wird nicht von %a, %b, %t, %c, %p untersttzt.

        buffer:     buffer != 0
        maxcount:   maxcount > 0
        format:     format != 0 / "Format-String" (s.z.B. printf und oben)
        argptr:     argptr != 0
        return:     Anzahl der in Buffer geschriebenen Zeichen (ohne '\0')
                    Ist Anzahl < 0 trat ein Fehler auf (normalerweise maxcount
                    zu klein). Inhalt von buffer ist dann undefiniert
\*---------------------------------------------------------------------------*/
int a_vsnprintf (char *buffer, const unsigned maxcount, const char *format, va_list argptr);
int a_snprintf (char *buffer, const unsigned maxcount, const char *format, ...);


/*---------------------------------------------------------------------------*\
    StrCmpTN
        Es werden die beiden Telefonnummern verglichen. Der Vergleich fngt am
        Ende des Strings an und vergleicht bis der Anfang des krzere Strings
        erreicht ist. Ignoriert werden die Zeichen: ' ' (SPACE), '(', ')', '+'
        tel1:       Die eine Telefonnummer (Darf == 0 sein!)
        tel2:       Die andere Telefonnummer (Darf == 0 sein!)
        return:     0 = identisch,  1 = NICHT identisch
\*---------------------------------------------------------------------------*/
unsigned StrCmpTN (const char *tel1, const char *tel2);


/*---------------------------------------------------------------------------*\
\*---------------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif

#endif
