/*---------------------------------------------------------------------------*\

    Copyright (C) 2000 AVM GmbH. All rights reserved.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY, without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, and WITHOUT
    ANY LIABILITY FOR ANY DAMAGES arising out of or in connection
    with the use or performance of this software. See the
    GNU General Public License for further details.

\*---------------------------------------------------------------------------*/

#include "TimerList.h"

/*---------------------------------------------------------------------------*\
 * CTimerSlot
\*---------------------------------------------------------------------------*/

class CTimerSlot {

public:
	CTimerSlot (CTimer * timer, unsigned target);
	~CTimerSlot (void);

	void NotifyTimer (void);

private:
	CTimerSlot *	link;
	CTimer *	timer;
	unsigned 	target;

	friend class CTimerList;
} ;

CTimerSlot::CTimerSlot (CTimer * timeR, unsigned taRget) :
	link (0)	,
	timer (timeR)	,
	target (taRget)
{
	// NOP
} // CTimerSlot::CTimerSlot

CTimerSlot::~CTimerSlot (void) {
	// NOP
} // CTimerSlot::~CTimerSlot

void CTimerSlot::NotifyTimer (void) {
	
	timer->HandleEvent ();
} // CTimerSlot::NotifyTimer

/*---------------------------------------------------------------------------*\
 * CTimerList
\*---------------------------------------------------------------------------*/

CTimerList::CTimerList (void) :
	list (0)	,
	count (0)
{
	// NOP
} // CTimerList::CTimerList

CTimerList::~CTimerList (void) {

	Clear ();
} // CTimerList::~CTimerList

void CTimerList::Add (CTimer * timer, unsigned time) {
	CTimerSlot * tmp1 = list;
	CTimerSlot * tmp2 = 0;
	CTimerSlot * new1 = new CTimerSlot (timer, time);

	while (tmp1 && (tmp1->target < time)) {
		tmp2 = tmp1;
		tmp1 = tmp1->link;
	}
	new1->link = tmp1;
	if (tmp2) {
		tmp2->link = new1;
	} else {
		list = new1;
	}
	++count;
} // CTimerList::Add

void CTimerList::Remove (CTimer * timer) {
	CTimerSlot * tmp1 = list;
	CTimerSlot * tmp2 = 0;

	while (tmp1 && (timer != tmp1->timer)) {
		tmp2 = tmp1;
		tmp1 = tmp1->link;
	}
	(void) internal_Remove (tmp1, tmp2);
} // CTimerList::Remove

void CTimerList::Clear (void) {
	CTimerSlot * temp = list;
	CTimerSlot * next;

	while (temp) {
		next = temp->link;
		delete temp;
		temp = next;
	}
	count = 0;
	list = 0;
} // CTimerList::Clear

void CTimerList::Notify (unsigned time) {
	CTimerSlot *	tmp1 = list;
	CTimerSlot *	tmp2 = 0;
	
	while (tmp1 && (tmp1->target != time)) {
		tmp2 = tmp1;
		tmp1 = tmp1->link;
	}
	while (tmp1 && (tmp1->target == time)) {
		tmp1->NotifyTimer ();
		tmp1 = internal_Remove (tmp1, tmp2);
	}
} // CTimerList::Notify

CTimerSlot * CTimerList::internal_Remove (CTimerSlot * s1, CTimerSlot * s2) {
	CTimerSlot * temp = 0;

	if (s1) {
		temp = s1->link;
		if (s2) {
			s2->link = s1->link;
		} else {
			list = s1->link;
		}
		delete s1;
		--count;
	}
	return temp;
} // CTimerList::internal_Remove

